/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsISOCKS4SocketInfo.idl
 */

#ifndef __gen_nsISOCKS4SocketInfo_h__
#define __gen_nsISOCKS4SocketInfo_h__


#ifndef __gen_nsIProxy_h__
#include "nsIProxy.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
#include "prtypes.h"
#include "prio.h"

/* starting interface:    nsISOCKS4SocketInfo */
#define NS_ISOCKS4SOCKETINFO_IID_STR "35b34e33-5756-4742-893a-d45d49a11942"

#define NS_ISOCKS4SOCKETINFO_IID \
  {0x35b34e33, 0x5756, 0x4742, \
    { 0x89, 0x3a, 0xd4, 0x5d, 0x49, 0xa1, 0x19, 0x42 }}

class NS_NO_VTABLE nsISOCKS4SocketInfo : public nsIProxy {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ISOCKS4SOCKETINFO_IID)

  /* attribute prNetAddr destinationAddr; */
  NS_IMETHOD GetDestinationAddr(PRNetAddr * *aDestinationAddr) = 0;
  NS_IMETHOD SetDestinationAddr(PRNetAddr * aDestinationAddr) = 0;

  /* attribute prNetAddr externalProxyAddr; */
  NS_IMETHOD GetExternalProxyAddr(PRNetAddr * *aExternalProxyAddr) = 0;
  NS_IMETHOD SetExternalProxyAddr(PRNetAddr * aExternalProxyAddr) = 0;

  /* attribute prNetAddr internalProxyAddr; */
  NS_IMETHOD GetInternalProxyAddr(PRNetAddr * *aInternalProxyAddr) = 0;
  NS_IMETHOD SetInternalProxyAddr(PRNetAddr * aInternalProxyAddr) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSISOCKS4SOCKETINFO \
  NS_IMETHOD GetDestinationAddr(PRNetAddr * *aDestinationAddr); \
  NS_IMETHOD SetDestinationAddr(PRNetAddr * aDestinationAddr); \
  NS_IMETHOD GetExternalProxyAddr(PRNetAddr * *aExternalProxyAddr); \
  NS_IMETHOD SetExternalProxyAddr(PRNetAddr * aExternalProxyAddr); \
  NS_IMETHOD GetInternalProxyAddr(PRNetAddr * *aInternalProxyAddr); \
  NS_IMETHOD SetInternalProxyAddr(PRNetAddr * aInternalProxyAddr); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSISOCKS4SOCKETINFO(_to) \
  NS_IMETHOD GetDestinationAddr(PRNetAddr * *aDestinationAddr) { return _to GetDestinationAddr(aDestinationAddr); } \
  NS_IMETHOD SetDestinationAddr(PRNetAddr * aDestinationAddr) { return _to SetDestinationAddr(aDestinationAddr); } \
  NS_IMETHOD GetExternalProxyAddr(PRNetAddr * *aExternalProxyAddr) { return _to GetExternalProxyAddr(aExternalProxyAddr); } \
  NS_IMETHOD SetExternalProxyAddr(PRNetAddr * aExternalProxyAddr) { return _to SetExternalProxyAddr(aExternalProxyAddr); } \
  NS_IMETHOD GetInternalProxyAddr(PRNetAddr * *aInternalProxyAddr) { return _to GetInternalProxyAddr(aInternalProxyAddr); } \
  NS_IMETHOD SetInternalProxyAddr(PRNetAddr * aInternalProxyAddr) { return _to SetInternalProxyAddr(aInternalProxyAddr); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSISOCKS4SOCKETINFO(_to) \
  NS_IMETHOD GetDestinationAddr(PRNetAddr * *aDestinationAddr) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetDestinationAddr(aDestinationAddr); } \
  NS_IMETHOD SetDestinationAddr(PRNetAddr * aDestinationAddr) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetDestinationAddr(aDestinationAddr); } \
  NS_IMETHOD GetExternalProxyAddr(PRNetAddr * *aExternalProxyAddr) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetExternalProxyAddr(aExternalProxyAddr); } \
  NS_IMETHOD SetExternalProxyAddr(PRNetAddr * aExternalProxyAddr) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetExternalProxyAddr(aExternalProxyAddr); } \
  NS_IMETHOD GetInternalProxyAddr(PRNetAddr * *aInternalProxyAddr) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetInternalProxyAddr(aInternalProxyAddr); } \
  NS_IMETHOD SetInternalProxyAddr(PRNetAddr * aInternalProxyAddr) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetInternalProxyAddr(aInternalProxyAddr); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsSOCKS4SocketInfo : public nsISOCKS4SocketInfo
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSISOCKS4SOCKETINFO

  nsSOCKS4SocketInfo();
  virtual ~nsSOCKS4SocketInfo();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsSOCKS4SocketInfo, nsISOCKS4SocketInfo)

nsSOCKS4SocketInfo::nsSOCKS4SocketInfo()
{
  /* member initializers and constructor code */
}

nsSOCKS4SocketInfo::~nsSOCKS4SocketInfo()
{
  /* destructor code */
}

/* attribute prNetAddr destinationAddr; */
NS_IMETHODIMP nsSOCKS4SocketInfo::GetDestinationAddr(PRNetAddr * *aDestinationAddr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsSOCKS4SocketInfo::SetDestinationAddr(PRNetAddr * aDestinationAddr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute prNetAddr externalProxyAddr; */
NS_IMETHODIMP nsSOCKS4SocketInfo::GetExternalProxyAddr(PRNetAddr * *aExternalProxyAddr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsSOCKS4SocketInfo::SetExternalProxyAddr(PRNetAddr * aExternalProxyAddr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute prNetAddr internalProxyAddr; */
NS_IMETHODIMP nsSOCKS4SocketInfo::GetInternalProxyAddr(PRNetAddr * *aInternalProxyAddr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsSOCKS4SocketInfo::SetInternalProxyAddr(PRNetAddr * aInternalProxyAddr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsISOCKS4SocketInfo_h__ */
