/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 2001 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Terry Hayes <thayes@netscape.com>
 */

#include "nsISupports.idl"

interface nsIInterfaceRequestor;

/**
 * nsICertPicker
 * Provides generic UI for choosing a certificate
 */
[scriptable, uuid(51d59b08-1dd2-11b2-ad4a-a51b92f8a184)]
interface nsICertPickDialogs : nsISupports
{
  /**
   * PickCertificate
   *   General purpose certificate prompter
   */
  void PickCertificate(in nsIInterfaceRequestor ctx, 
                       [array, size_is(count)] in wstring certNickList,
                       [array, size_is(count)] in wstring certDetailsList,
                       in unsigned long count,
                       inout long selectedIndex,
                       out boolean canceled);
};

%{C++
#define NS_CERTPICKDIALOGS_CONTRACTID "@mozilla.org/nsCertPickDialogs;1"
%}
