/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIPop3IncomingServer.idl
 */

#ifndef __gen_nsIPop3IncomingServer_h__
#define __gen_nsIPop3IncomingServer_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIPop3IncomingServer */
#define NS_IPOP3INCOMINGSERVER_IID_STR "758a8970-e628-11d2-b7fc-00805f05ffa5"

#define NS_IPOP3INCOMINGSERVER_IID \
  {0x758a8970, 0xe628, 0x11d2, \
    { 0xb7, 0xfc, 0x00, 0x80, 0x5f, 0x05, 0xff, 0xa5 }}

class NS_NO_VTABLE nsIPop3IncomingServer : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IPOP3INCOMINGSERVER_IID)

  /* attribute boolean leaveMessagesOnServer; */
  NS_IMETHOD GetLeaveMessagesOnServer(PRBool *aLeaveMessagesOnServer) = 0;
  NS_IMETHOD SetLeaveMessagesOnServer(PRBool aLeaveMessagesOnServer) = 0;

  /* attribute boolean deleteMailLeftOnServer; */
  NS_IMETHOD GetDeleteMailLeftOnServer(PRBool *aDeleteMailLeftOnServer) = 0;
  NS_IMETHOD SetDeleteMailLeftOnServer(PRBool aDeleteMailLeftOnServer) = 0;

  /* attribute boolean authLogin; */
  NS_IMETHOD GetAuthLogin(PRBool *aAuthLogin) = 0;
  NS_IMETHOD SetAuthLogin(PRBool aAuthLogin) = 0;

  /* attribute boolean dotFix; */
  NS_IMETHOD GetDotFix(PRBool *aDotFix) = 0;
  NS_IMETHOD SetDotFix(PRBool aDotFix) = 0;

  /* attribute unsigned long pop3CapabilityFlags; */
  NS_IMETHOD GetPop3CapabilityFlags(PRUint32 *aPop3CapabilityFlags) = 0;
  NS_IMETHOD SetPop3CapabilityFlags(PRUint32 aPop3CapabilityFlags) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIPOP3INCOMINGSERVER \
  NS_IMETHOD GetLeaveMessagesOnServer(PRBool *aLeaveMessagesOnServer); \
  NS_IMETHOD SetLeaveMessagesOnServer(PRBool aLeaveMessagesOnServer); \
  NS_IMETHOD GetDeleteMailLeftOnServer(PRBool *aDeleteMailLeftOnServer); \
  NS_IMETHOD SetDeleteMailLeftOnServer(PRBool aDeleteMailLeftOnServer); \
  NS_IMETHOD GetAuthLogin(PRBool *aAuthLogin); \
  NS_IMETHOD SetAuthLogin(PRBool aAuthLogin); \
  NS_IMETHOD GetDotFix(PRBool *aDotFix); \
  NS_IMETHOD SetDotFix(PRBool aDotFix); \
  NS_IMETHOD GetPop3CapabilityFlags(PRUint32 *aPop3CapabilityFlags); \
  NS_IMETHOD SetPop3CapabilityFlags(PRUint32 aPop3CapabilityFlags); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIPOP3INCOMINGSERVER(_to) \
  NS_IMETHOD GetLeaveMessagesOnServer(PRBool *aLeaveMessagesOnServer) { return _to GetLeaveMessagesOnServer(aLeaveMessagesOnServer); } \
  NS_IMETHOD SetLeaveMessagesOnServer(PRBool aLeaveMessagesOnServer) { return _to SetLeaveMessagesOnServer(aLeaveMessagesOnServer); } \
  NS_IMETHOD GetDeleteMailLeftOnServer(PRBool *aDeleteMailLeftOnServer) { return _to GetDeleteMailLeftOnServer(aDeleteMailLeftOnServer); } \
  NS_IMETHOD SetDeleteMailLeftOnServer(PRBool aDeleteMailLeftOnServer) { return _to SetDeleteMailLeftOnServer(aDeleteMailLeftOnServer); } \
  NS_IMETHOD GetAuthLogin(PRBool *aAuthLogin) { return _to GetAuthLogin(aAuthLogin); } \
  NS_IMETHOD SetAuthLogin(PRBool aAuthLogin) { return _to SetAuthLogin(aAuthLogin); } \
  NS_IMETHOD GetDotFix(PRBool *aDotFix) { return _to GetDotFix(aDotFix); } \
  NS_IMETHOD SetDotFix(PRBool aDotFix) { return _to SetDotFix(aDotFix); } \
  NS_IMETHOD GetPop3CapabilityFlags(PRUint32 *aPop3CapabilityFlags) { return _to GetPop3CapabilityFlags(aPop3CapabilityFlags); } \
  NS_IMETHOD SetPop3CapabilityFlags(PRUint32 aPop3CapabilityFlags) { return _to SetPop3CapabilityFlags(aPop3CapabilityFlags); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIPOP3INCOMINGSERVER(_to) \
  NS_IMETHOD GetLeaveMessagesOnServer(PRBool *aLeaveMessagesOnServer) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLeaveMessagesOnServer(aLeaveMessagesOnServer); } \
  NS_IMETHOD SetLeaveMessagesOnServer(PRBool aLeaveMessagesOnServer) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetLeaveMessagesOnServer(aLeaveMessagesOnServer); } \
  NS_IMETHOD GetDeleteMailLeftOnServer(PRBool *aDeleteMailLeftOnServer) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetDeleteMailLeftOnServer(aDeleteMailLeftOnServer); } \
  NS_IMETHOD SetDeleteMailLeftOnServer(PRBool aDeleteMailLeftOnServer) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetDeleteMailLeftOnServer(aDeleteMailLeftOnServer); } \
  NS_IMETHOD GetAuthLogin(PRBool *aAuthLogin) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetAuthLogin(aAuthLogin); } \
  NS_IMETHOD SetAuthLogin(PRBool aAuthLogin) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetAuthLogin(aAuthLogin); } \
  NS_IMETHOD GetDotFix(PRBool *aDotFix) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetDotFix(aDotFix); } \
  NS_IMETHOD SetDotFix(PRBool aDotFix) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetDotFix(aDotFix); } \
  NS_IMETHOD GetPop3CapabilityFlags(PRUint32 *aPop3CapabilityFlags) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetPop3CapabilityFlags(aPop3CapabilityFlags); } \
  NS_IMETHOD SetPop3CapabilityFlags(PRUint32 aPop3CapabilityFlags) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetPop3CapabilityFlags(aPop3CapabilityFlags); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsPop3IncomingServer : public nsIPop3IncomingServer
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIPOP3INCOMINGSERVER

  nsPop3IncomingServer();
  virtual ~nsPop3IncomingServer();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsPop3IncomingServer, nsIPop3IncomingServer)

nsPop3IncomingServer::nsPop3IncomingServer()
{
  /* member initializers and constructor code */
}

nsPop3IncomingServer::~nsPop3IncomingServer()
{
  /* destructor code */
}

/* attribute boolean leaveMessagesOnServer; */
NS_IMETHODIMP nsPop3IncomingServer::GetLeaveMessagesOnServer(PRBool *aLeaveMessagesOnServer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsPop3IncomingServer::SetLeaveMessagesOnServer(PRBool aLeaveMessagesOnServer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute boolean deleteMailLeftOnServer; */
NS_IMETHODIMP nsPop3IncomingServer::GetDeleteMailLeftOnServer(PRBool *aDeleteMailLeftOnServer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsPop3IncomingServer::SetDeleteMailLeftOnServer(PRBool aDeleteMailLeftOnServer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute boolean authLogin; */
NS_IMETHODIMP nsPop3IncomingServer::GetAuthLogin(PRBool *aAuthLogin)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsPop3IncomingServer::SetAuthLogin(PRBool aAuthLogin)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute boolean dotFix; */
NS_IMETHODIMP nsPop3IncomingServer::GetDotFix(PRBool *aDotFix)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsPop3IncomingServer::SetDotFix(PRBool aDotFix)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute unsigned long pop3CapabilityFlags; */
NS_IMETHODIMP nsPop3IncomingServer::GetPop3CapabilityFlags(PRUint32 *aPop3CapabilityFlags)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsPop3IncomingServer::SetPop3CapabilityFlags(PRUint32 aPop3CapabilityFlags)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIPop3IncomingServer_h__ */
