/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIMsgFolderCache.idl
 */

#ifndef __gen_nsIMsgFolderCache_h__
#define __gen_nsIMsgFolderCache_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIFileSpec; /* forward declaration */

class nsIMsgFolderCacheElement; /* forward declaration */


/* starting interface:    nsIMsgFolderCache */
#define NS_IMSGFOLDERCACHE_IID_STR "d8a130f0-397d-11d3-8d76-00805f8a6617"

#define NS_IMSGFOLDERCACHE_IID \
  {0xd8a130f0, 0x397d, 0x11d3, \
    { 0x8d, 0x76, 0x00, 0x80, 0x5f, 0x8a, 0x66, 0x17 }}

class NS_NO_VTABLE nsIMsgFolderCache : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMSGFOLDERCACHE_IID)

  /* void Init (in nsIFileSpec fileSpec); */
  NS_IMETHOD Init(nsIFileSpec *fileSpec) = 0;

  /* nsIMsgFolderCacheElement GetCacheElement (in string key, in boolean createIfMissing); */
  NS_IMETHOD GetCacheElement(const char *key, PRBool createIfMissing, nsIMsgFolderCacheElement **_retval) = 0;

  /* void Close (); */
  NS_IMETHOD Close(void) = 0;

  /* void Commit (in boolean compress); */
  NS_IMETHOD Commit(PRBool compress) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMSGFOLDERCACHE \
  NS_IMETHOD Init(nsIFileSpec *fileSpec); \
  NS_IMETHOD GetCacheElement(const char *key, PRBool createIfMissing, nsIMsgFolderCacheElement **_retval); \
  NS_IMETHOD Close(void); \
  NS_IMETHOD Commit(PRBool compress); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMSGFOLDERCACHE(_to) \
  NS_IMETHOD Init(nsIFileSpec *fileSpec) { return _to Init(fileSpec); } \
  NS_IMETHOD GetCacheElement(const char *key, PRBool createIfMissing, nsIMsgFolderCacheElement **_retval) { return _to GetCacheElement(key, createIfMissing, _retval); } \
  NS_IMETHOD Close(void) { return _to Close(); } \
  NS_IMETHOD Commit(PRBool compress) { return _to Commit(compress); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIMSGFOLDERCACHE(_to) \
  NS_IMETHOD Init(nsIFileSpec *fileSpec) { return !_to ? NS_ERROR_NULL_POINTER : _to->Init(fileSpec); } \
  NS_IMETHOD GetCacheElement(const char *key, PRBool createIfMissing, nsIMsgFolderCacheElement **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetCacheElement(key, createIfMissing, _retval); } \
  NS_IMETHOD Close(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->Close(); } \
  NS_IMETHOD Commit(PRBool compress) { return !_to ? NS_ERROR_NULL_POINTER : _to->Commit(compress); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMsgFolderCache : public nsIMsgFolderCache
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMSGFOLDERCACHE

  nsMsgFolderCache();
  virtual ~nsMsgFolderCache();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsMsgFolderCache, nsIMsgFolderCache)

nsMsgFolderCache::nsMsgFolderCache()
{
  /* member initializers and constructor code */
}

nsMsgFolderCache::~nsMsgFolderCache()
{
  /* destructor code */
}

/* void Init (in nsIFileSpec fileSpec); */
NS_IMETHODIMP nsMsgFolderCache::Init(nsIFileSpec *fileSpec)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIMsgFolderCacheElement GetCacheElement (in string key, in boolean createIfMissing); */
NS_IMETHODIMP nsMsgFolderCache::GetCacheElement(const char *key, PRBool createIfMissing, nsIMsgFolderCacheElement **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void Close (); */
NS_IMETHODIMP nsMsgFolderCache::Close()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void Commit (in boolean compress); */
NS_IMETHODIMP nsMsgFolderCache::Commit(PRBool compress)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMsgFolderCache_h__ */
