/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM inIBitmap.idl
 */

#ifndef __gen_inIBitmap_h__
#define __gen_inIBitmap_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    inIBitmap */
#define INIBITMAP_IID_STR "bacf8fb9-1fe0-40f0-852c-193442b75c0f"

#define INIBITMAP_IID \
  {0xbacf8fb9, 0x1fe0, 0x40f0, \
    { 0x85, 0x2c, 0x19, 0x34, 0x42, 0xb7, 0x5c, 0x0f }}

class NS_NO_VTABLE inIBitmap : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(INIBITMAP_IID)

  /* readonly attribute unsigned long width; */
  NS_IMETHOD GetWidth(PRUint32 *aWidth) = 0;

  /* readonly attribute unsigned long height; */
  NS_IMETHOD GetHeight(PRUint32 *aHeight) = 0;

  /* void init (in unsigned long aWidth, in unsigned long aHeight, in octet aBytesPerPixel); */
  NS_IMETHOD Init(PRUint32 aWidth, PRUint32 aHeight, PRUint8 aBytesPerPixel) = 0;

  /* wstring getPixelHex (in unsigned long aX, in unsigned long aY); */
  NS_IMETHOD GetPixelHex(PRUint32 aX, PRUint32 aY, PRUnichar **_retval) = 0;

  /* wstring putPixel (in unsigned long aX, in unsigned long aY, in octet aR, in octet aG, in octet aB); */
  NS_IMETHOD PutPixel(PRUint32 aX, PRUint32 aY, PRUint8 aR, PRUint8 aG, PRUint8 aB, PRUnichar **_retval) = 0;

  /* wstring putPixelHex (in unsigned long aX, in unsigned long aY, in wstring aColor); */
  NS_IMETHOD PutPixelHex(PRUint32 aX, PRUint32 aY, const PRUnichar *aColor, PRUnichar **_retval) = 0;

  /* [noscript] void GetBits (out nativePRUint8 aBits); */
  NS_IMETHOD GetBits(PRUint8 * *aBits) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_INIBITMAP \
  NS_IMETHOD GetWidth(PRUint32 *aWidth); \
  NS_IMETHOD GetHeight(PRUint32 *aHeight); \
  NS_IMETHOD Init(PRUint32 aWidth, PRUint32 aHeight, PRUint8 aBytesPerPixel); \
  NS_IMETHOD GetPixelHex(PRUint32 aX, PRUint32 aY, PRUnichar **_retval); \
  NS_IMETHOD PutPixel(PRUint32 aX, PRUint32 aY, PRUint8 aR, PRUint8 aG, PRUint8 aB, PRUnichar **_retval); \
  NS_IMETHOD PutPixelHex(PRUint32 aX, PRUint32 aY, const PRUnichar *aColor, PRUnichar **_retval); \
  NS_IMETHOD GetBits(PRUint8 * *aBits); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_INIBITMAP(_to) \
  NS_IMETHOD GetWidth(PRUint32 *aWidth) { return _to GetWidth(aWidth); } \
  NS_IMETHOD GetHeight(PRUint32 *aHeight) { return _to GetHeight(aHeight); } \
  NS_IMETHOD Init(PRUint32 aWidth, PRUint32 aHeight, PRUint8 aBytesPerPixel) { return _to Init(aWidth, aHeight, aBytesPerPixel); } \
  NS_IMETHOD GetPixelHex(PRUint32 aX, PRUint32 aY, PRUnichar **_retval) { return _to GetPixelHex(aX, aY, _retval); } \
  NS_IMETHOD PutPixel(PRUint32 aX, PRUint32 aY, PRUint8 aR, PRUint8 aG, PRUint8 aB, PRUnichar **_retval) { return _to PutPixel(aX, aY, aR, aG, aB, _retval); } \
  NS_IMETHOD PutPixelHex(PRUint32 aX, PRUint32 aY, const PRUnichar *aColor, PRUnichar **_retval) { return _to PutPixelHex(aX, aY, aColor, _retval); } \
  NS_IMETHOD GetBits(PRUint8 * *aBits) { return _to GetBits(aBits); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_INIBITMAP(_to) \
  NS_IMETHOD GetWidth(PRUint32 *aWidth) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetWidth(aWidth); } \
  NS_IMETHOD GetHeight(PRUint32 *aHeight) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetHeight(aHeight); } \
  NS_IMETHOD Init(PRUint32 aWidth, PRUint32 aHeight, PRUint8 aBytesPerPixel) { return !_to ? NS_ERROR_NULL_POINTER : _to->Init(aWidth, aHeight, aBytesPerPixel); } \
  NS_IMETHOD GetPixelHex(PRUint32 aX, PRUint32 aY, PRUnichar **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetPixelHex(aX, aY, _retval); } \
  NS_IMETHOD PutPixel(PRUint32 aX, PRUint32 aY, PRUint8 aR, PRUint8 aG, PRUint8 aB, PRUnichar **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->PutPixel(aX, aY, aR, aG, aB, _retval); } \
  NS_IMETHOD PutPixelHex(PRUint32 aX, PRUint32 aY, const PRUnichar *aColor, PRUnichar **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->PutPixelHex(aX, aY, aColor, _retval); } \
  NS_IMETHOD GetBits(PRUint8 * *aBits) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetBits(aBits); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class inBitmap : public inIBitmap
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_INIBITMAP

  inBitmap();
  virtual ~inBitmap();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(inBitmap, inIBitmap)

inBitmap::inBitmap()
{
  /* member initializers and constructor code */
}

inBitmap::~inBitmap()
{
  /* destructor code */
}

/* readonly attribute unsigned long width; */
NS_IMETHODIMP inBitmap::GetWidth(PRUint32 *aWidth)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long height; */
NS_IMETHODIMP inBitmap::GetHeight(PRUint32 *aHeight)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void init (in unsigned long aWidth, in unsigned long aHeight, in octet aBytesPerPixel); */
NS_IMETHODIMP inBitmap::Init(PRUint32 aWidth, PRUint32 aHeight, PRUint8 aBytesPerPixel)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* wstring getPixelHex (in unsigned long aX, in unsigned long aY); */
NS_IMETHODIMP inBitmap::GetPixelHex(PRUint32 aX, PRUint32 aY, PRUnichar **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* wstring putPixel (in unsigned long aX, in unsigned long aY, in octet aR, in octet aG, in octet aB); */
NS_IMETHODIMP inBitmap::PutPixel(PRUint32 aX, PRUint32 aY, PRUint8 aR, PRUint8 aG, PRUint8 aB, PRUnichar **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* wstring putPixelHex (in unsigned long aX, in unsigned long aY, in wstring aColor); */
NS_IMETHODIMP inBitmap::PutPixelHex(PRUint32 aX, PRUint32 aY, const PRUnichar *aColor, PRUnichar **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] void GetBits (out nativePRUint8 aBits); */
NS_IMETHODIMP inBitmap::GetBits(PRUint8 * *aBits)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_inIBitmap_h__ */
