/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 *   Radha Kulkarni <radha@netscape.com>
 */

#include "nsISupports.idl"
#include "nsISHEntry.idl"
#include "nsIEnumerator.idl"


/**
 * The nsISHTransaction.
 */


[scriptable, uuid(2F0C3A72-988B-11d3-BDC7-0050040A9B44)]
interface nsISHTransaction : nsISupports
{
	/**
     * The nsISHEntry for the current transaction
     */
	attribute nsISHEntry sHEntry;

	/**
	 * The parent of this transaction
	 */
	attribute nsISHTransaction prev;

	/**
	 * The legitimate child of this transaction 
	 */
	attribute nsISHTransaction next;

	/**
	 * The other paths that have been navigated from this transaction
	 */
	readonly attribute nsISHTransaction lrvList;

	/**
	 *	Specifies if this transaction should persist.  If not it will be replaced
	 * by new additions to the list.
	 */
	attribute boolean persist;

	/**
	 * Create a transaction with parent and History Entry 
	 */
	 void create(in nsISHEntry aSHEntry, in nsISHTransaction aPrev);
};

%{ C++
// {BFD1A792-AD9F-11d3-BDC7-0050040A9B44}


#define NS_SHTRANSACTION_CID \
{0xbfd1a792, 0xad9f, 0x11d3, {0xbd, 0xc7, 0x0, 0x50, 0x4, 0xa, 0x9b, 0x44}}

#define NS_SHTRANSACTION_CONTRACTID \
    "@mozilla.org/browser/session-history-transaction;1"

%}
