/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIMsgVCard.idl
 */

#ifndef __gen_nsIMsgVCard_h__
#define __gen_nsIMsgVCard_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIMsgVCard */
#define NS_IMSGVCARD_IID_STR "e0e67ec0-e3df-11d2-b7fc-00805f05ffa5"

#define NS_IMSGVCARD_IID \
  {0xe0e67ec0, 0xe3df, 0x11d2, \
    { 0xb7, 0xfc, 0x00, 0x80, 0x5f, 0x05, 0xff, 0xa5 }}

class NS_NO_VTABLE nsIMsgVCard : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMSGVCARD_IID)

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMSGVCARD \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMSGVCARD(_to) \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIMSGVCARD(_to) \
  /* no methods! */

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMsgVCard : public nsIMsgVCard
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMSGVCARD

  nsMsgVCard();
  virtual ~nsMsgVCard();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsMsgVCard, nsIMsgVCard)

nsMsgVCard::nsMsgVCard()
{
  /* member initializers and constructor code */
}

nsMsgVCard::~nsMsgVCard()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMsgVCard_h__ */
