/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Mozilla Communicator.
 * 
 * The Initial Developer of the Original Code is Netscape Communications
 * Corp.  Portions created by Netscape are Copyright (C) 1999 Netscape 
 * Communications Corp.  All Rights Reserved.
 * 
 * Contributor(s): 
 *   Mike Pinkerton
 */

#include "nsISupports.idl"


/*
 * nsIDragTracker
 *
 * An interface used with the drag service/session to be notified 
 * when a drag is over or out of its window. This allows an object to,
 * for example, grab the mouse until and be told when it should stop.
 * 
 * When the joyride is over, the Drag Session calls |stopTracking()|
 * to alert the object that it should stop watching.
 */
 
[scriptable, uuid(641efb5c-1dd2-11b2-9067-8020880c3961)]
interface nsIDragTracker : nsISupports
{  
  /**
    * stopTracking ( )
    *
    * Call to tell the object that is tracking the drag that it
    * can forget about it (mouse left the window or the drop completed).
    */
  void stopTracking ( ) ;
  
};


%{ C++

%}
