/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM inIBitmapURI.idl
 */

#ifndef __gen_inIBitmapURI_h__
#define __gen_inIBitmapURI_h__


#ifndef __gen_nsIURI_h__
#include "nsIURI.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    inIBitmapURI */
#define INIBITMAPURI_IID_STR "5f0d8f70-15a9-4c4d-b952-086005a8133c"

#define INIBITMAPURI_IID \
  {0x5f0d8f70, 0x15a9, 0x4c4d, \
    { 0xb9, 0x52, 0x08, 0x60, 0x05, 0xa8, 0x13, 0x3c }}

class NS_NO_VTABLE inIBitmapURI : public nsIURI {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(INIBITMAPURI_IID)

  /* readonly attribute wstring bitmapName; */
  NS_IMETHOD GetBitmapName(PRUnichar * *aBitmapName) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_INIBITMAPURI \
  NS_IMETHOD GetBitmapName(PRUnichar * *aBitmapName); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_INIBITMAPURI(_to) \
  NS_IMETHOD GetBitmapName(PRUnichar * *aBitmapName) { return _to GetBitmapName(aBitmapName); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_INIBITMAPURI(_to) \
  NS_IMETHOD GetBitmapName(PRUnichar * *aBitmapName) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetBitmapName(aBitmapName); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class inBitmapURI : public inIBitmapURI
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_INIBITMAPURI

  inBitmapURI();
  virtual ~inBitmapURI();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(inBitmapURI, inIBitmapURI)

inBitmapURI::inBitmapURI()
{
  /* member initializers and constructor code */
}

inBitmapURI::~inBitmapURI()
{
  /* destructor code */
}

/* readonly attribute wstring bitmapName; */
NS_IMETHODIMP inBitmapURI::GetBitmapName(PRUnichar * *aBitmapName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_inIBitmapURI_h__ */
