/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsISOCKSSocketProvider.idl
 */

#ifndef __gen_nsISOCKSSocketProvider_h__
#define __gen_nsISOCKSSocketProvider_h__


#ifndef __gen_nsISocketProvider_h__
#include "nsISocketProvider.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsISOCKSSocketProvider */
#define NS_ISOCKSSOCKETPROVIDER_IID_STR "8dbe7246-1dd2-11b2-9b8f-b9a849e4403a"

#define NS_ISOCKSSOCKETPROVIDER_IID \
  {0x8dbe7246, 0x1dd2, 0x11b2, \
    { 0x9b, 0x8f, 0xb9, 0xa8, 0x49, 0xe4, 0x40, 0x3a }}

class NS_NO_VTABLE nsISOCKSSocketProvider : public nsISocketProvider {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ISOCKSSOCKETPROVIDER_IID)

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSISOCKSSOCKETPROVIDER \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSISOCKSSOCKETPROVIDER(_to) \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSISOCKSSOCKETPROVIDER(_to) \
  /* no methods! */

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsSOCKSSocketProvider : public nsISOCKSSocketProvider
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSISOCKSSOCKETPROVIDER

  nsSOCKSSocketProvider();
  virtual ~nsSOCKSSocketProvider();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsSOCKSSocketProvider, nsISOCKSSocketProvider)

nsSOCKSSocketProvider::nsSOCKSSocketProvider()
{
  /* member initializers and constructor code */
}

nsSOCKSSocketProvider::~nsSOCKSSocketProvider()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif

#define NS_ISOCKSSOCKETPROVIDER_CONTRACTID		NS_NETWORK_SOCKET_CONTRACTID_PREFIX "socks"
#define NS_ISOCKSSOCKETPROVIDER_CLASSNAME	"Mozilla SOCKS Socket Provider Component"

#endif /* __gen_nsISOCKSSocketProvider_h__ */
