/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsrootidl.idl"
#include "nsIMsgStatusFeedback.idl"

interface nsIDOMWindowInternal;
interface nsIPrintSettings;
interface nsIObserver;
interface nsIWebBrowserPrint;

[scriptable, uuid(91FD6B10-E0BC-11d3-8F97-000064657374)]
interface nsIMsgPrintEngine : nsISupports {
    /**
     * Print/PrintPreview Msg Type
     */
    const short MNAB_START                 = 0;
    const short MNAB_PRINT_MSG             = 0;
    const short MNAB_PRINTPREVIEW_MSG      = 1;
    const short MNAB_PRINT_AB_CARD         = 2;
    const short MNAB_PRINTPREVIEW_AB_CARD  = 3;
    const short MNAB_PRINT_ADDRBOOK        = 4;
    const short MNAB_PRINTPREVIEW_ADDRBOOK = 5;
    const short MNAB_END                   = 6;

    void setWindow(in nsIDOMWindowInternal ptr);
    void setParentWindow(in nsIDOMWindowInternal ptr);
    void showWindow(in boolean aShow);
    void setStatusFeedback(in nsIMsgStatusFeedback feedback);
    void setPrintURICount(in PRInt32 aCount);
    void addPrintURI(in wstring aURI);
    void startPrintOperation(in nsIPrintSettings aPS);
    void setStartupPPObserver(in nsIObserver startupPPObs);
    void setMsgType(in long aMsgType);

    attribute boolean doPrintPreview;
    readonly attribute nsIWebBrowserPrint webBrowserPrint;

};

