/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "nsISupports.idl"

interface nsIContentViewer;

[scriptable, uuid(ea2ce7a0-5c3d-11d4-90c2-0050041caf44)]
interface nsIContentViewerContainer : nsISupports {
	void embed(in nsIContentViewer aDocViewer, in string aCommand, in nsISupports aExtraInfo);

  /**
   * Allows the PrintEngine to make this call on 
   * an internal interface to the DocShell
   */
  void setIsPrinting(in boolean aIsPrinting);
};
