/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsINetModRegEntry.idl
 */

#ifndef __gen_nsINetModRegEntry_h__
#define __gen_nsINetModRegEntry_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsINetNotify_h__
#include "nsINetNotify.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIEventQueue; /* forward declaration */

class nsINetModRegEntry; /* forward declaration */

class nsINetNotify; /* forward declaration */

// {F126BD90-1472-11d3-A15A-0050041CAF44}
#define NS_NETMODREGENTRY_CID \
{ 0xf126bd90, 0x1472, 0x11d3, { 0xa1, 0x5a, 0x0, 0x50, 0x4, 0x1c, 0xaf, 0x44 } }

/* starting interface:    nsINetModRegEntry */
#define NS_INETMODREGENTRY_IID_STR "9f482bd0-1476-11d3-a15a-0050041caf44"

#define NS_INETMODREGENTRY_IID \
  {0x9f482bd0, 0x1476, 0x11d3, \
    { 0xa1, 0x5a, 0x00, 0x50, 0x04, 0x1c, 0xaf, 0x44 }}

class NS_NO_VTABLE nsINetModRegEntry : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_INETMODREGENTRY_IID)

  /* readonly attribute nsINetNotify syncProxy; */
  NS_IMETHOD GetSyncProxy(nsINetNotify * *aSyncProxy) = 0;

  /* readonly attribute nsINetNotify asyncProxy; */
  NS_IMETHOD GetAsyncProxy(nsINetNotify * *aAsyncProxy) = 0;

  /* readonly attribute string topic; */
  NS_IMETHOD GetTopic(char * *aTopic) = 0;

  /* boolean equals (in nsINetModRegEntry aEntry); */
  NS_IMETHOD Equals(nsINetModRegEntry *aEntry, PRBool *_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSINETMODREGENTRY \
  NS_IMETHOD GetSyncProxy(nsINetNotify * *aSyncProxy); \
  NS_IMETHOD GetAsyncProxy(nsINetNotify * *aAsyncProxy); \
  NS_IMETHOD GetTopic(char * *aTopic); \
  NS_IMETHOD Equals(nsINetModRegEntry *aEntry, PRBool *_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSINETMODREGENTRY(_to) \
  NS_IMETHOD GetSyncProxy(nsINetNotify * *aSyncProxy) { return _to GetSyncProxy(aSyncProxy); } \
  NS_IMETHOD GetAsyncProxy(nsINetNotify * *aAsyncProxy) { return _to GetAsyncProxy(aAsyncProxy); } \
  NS_IMETHOD GetTopic(char * *aTopic) { return _to GetTopic(aTopic); } \
  NS_IMETHOD Equals(nsINetModRegEntry *aEntry, PRBool *_retval) { return _to Equals(aEntry, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSINETMODREGENTRY(_to) \
  NS_IMETHOD GetSyncProxy(nsINetNotify * *aSyncProxy) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSyncProxy(aSyncProxy); } \
  NS_IMETHOD GetAsyncProxy(nsINetNotify * *aAsyncProxy) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetAsyncProxy(aAsyncProxy); } \
  NS_IMETHOD GetTopic(char * *aTopic) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetTopic(aTopic); } \
  NS_IMETHOD Equals(nsINetModRegEntry *aEntry, PRBool *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->Equals(aEntry, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsNetModRegEntry : public nsINetModRegEntry
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSINETMODREGENTRY

  nsNetModRegEntry();
  virtual ~nsNetModRegEntry();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsNetModRegEntry, nsINetModRegEntry)

nsNetModRegEntry::nsNetModRegEntry()
{
  /* member initializers and constructor code */
}

nsNetModRegEntry::~nsNetModRegEntry()
{
  /* destructor code */
}

/* readonly attribute nsINetNotify syncProxy; */
NS_IMETHODIMP nsNetModRegEntry::GetSyncProxy(nsINetNotify * *aSyncProxy)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsINetNotify asyncProxy; */
NS_IMETHODIMP nsNetModRegEntry::GetAsyncProxy(nsINetNotify * *aAsyncProxy)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute string topic; */
NS_IMETHODIMP nsNetModRegEntry::GetTopic(char * *aTopic)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* boolean equals (in nsINetModRegEntry aEntry); */
NS_IMETHODIMP nsNetModRegEntry::Equals(nsINetModRegEntry *aEntry, PRBool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsINetModRegEntry_h__ */
