/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Blake Ross <blaker@netscape.com> 
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIURI;
interface nsILocalFile;
interface nsIObserver;
interface nsIWebBrowserPersist;
interface nsIWebProgressListener;
interface nsIMIMEInfo;

[scriptable, uuid(06cb92f2-1dd2-11b2-95f2-96dfdfb804a1)]
interface nsIDownload : nsISupports {
    
    /**
     * Initializes the download with certain properties.
     *
     * @param aSource The source (nsIURI) of the download.
     *
     * @param aTarget The local file to which the download is being saved.
     *
     * @param aDisplayName The user-readable description of the download.
     *
     * @param aMIMEInfo The MIME info associated with the download file,
     *                  including MIME type and helper app when appropriate.
     *                  This parameter is optional.
     *
     * @param aPersist The "persist" used to transfer the download.  If set,
     *                 the manager will set its listener to the download item
     *                 and use it for cancellation.  If not set, the client
     *                 is expected to set the download item as the listener on
     *                 whatever transfer component is being used, and to
     *                 set an observer on the download item that listens for
     *                 the "oncancel" topic and cancels the download.
     */
    void init(in nsIURI aSource,
              in nsILocalFile aTarget,
              in wstring aDisplayName,
              in nsIMIMEInfo aMIMEInfo,
              in long long startTime,
              in nsIWebBrowserPersist aPersist);

    /**
     * The source of the download.
     */
    readonly attribute nsIURI source;
    
    /**
     * The local file to which the download is being saved.
     */
    readonly attribute nsILocalFile target;
 
    /**
     * Optional. If set, it will be used for cancellation, and the download
     * will be set as its listener.  If not, |observer| should be set to listen
     * and respond accordingly to topics like oncancel, and the client promises
     * to set the download item as the listener for whatever transfer component
     * being used.
     */
 
    readonly attribute nsIWebBrowserPersist persist;
   
    /**
     * The percentage of completion of the download.
     */
    readonly attribute PRInt32 percentComplete;

    /**
     * The user-readable description of the download.
     */
    attribute wstring displayName;

    /**
     * The time a download was started.
     */
    readonly attribute long long startTime;
    
    /**
     * Optional. If set, it will contain the download's relevant MIME information.
     * This includes it's MIME Type, helper app, and whether that helper should be
     * executed.
     */
    readonly attribute nsIMIMEInfo MIMEInfo;

    /**
     * Optional; downloading information is passed to this listener and used to
     * update client UI.
     */
    attribute nsIWebProgressListener listener;
 
    /**
     * If set, receives notifications of events like cancel ("oncancel").
     * Must be set if no persist object is specified (see above).
     */
    attribute nsIObserver observer;
};

%{C++
#define NS_DOWNLOAD_CONTRACTID "@mozilla.org/download;1"
// {E3FA9D0A-1DD1-11B2-BDEF-8C720B597445}
#define NS_DOWNLOAD_CID \
    { 0xe3fa9d0a, 0x1dd1, 0x11b2, { 0xbd, 0xef, 0x8c, 0x72, 0x0b, 0x59, 0x74, 0x45 } }
%}
