/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIPasswordSink.idl
 */

#ifndef __gen_nsIPasswordSink_h__
#define __gen_nsIPasswordSink_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIPasswordSink */
#define NS_IPASSWORDSINK_IID_STR "576e715a-1dd2-11b2-80be-8c835a34d50b"

#define NS_IPASSWORDSINK_IID \
  {0x576e715a, 0x1dd2, 0x11b2, \
    { 0x80, 0xbe, 0x8c, 0x83, 0x5a, 0x34, 0xd5, 0x0b }}

class NS_NO_VTABLE nsIPasswordSink : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IPASSWORDSINK_IID)

  /* readonly attribute wstring password; */
  NS_IMETHOD GetPassword(PRUnichar * *aPassword) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIPASSWORDSINK \
  NS_IMETHOD GetPassword(PRUnichar * *aPassword); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIPASSWORDSINK(_to) \
  NS_IMETHOD GetPassword(PRUnichar * *aPassword) { return _to GetPassword(aPassword); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIPASSWORDSINK(_to) \
  NS_IMETHOD GetPassword(PRUnichar * *aPassword) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetPassword(aPassword); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsPasswordSink : public nsIPasswordSink
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIPASSWORDSINK

  nsPasswordSink();
  virtual ~nsPasswordSink();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsPasswordSink, nsIPasswordSink)

nsPasswordSink::nsPasswordSink()
{
  /* member initializers and constructor code */
}

nsPasswordSink::~nsPasswordSink()
{
  /* destructor code */
}

/* readonly attribute wstring password; */
NS_IMETHODIMP nsPasswordSink::GetPassword(PRUnichar * *aPassword)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIPasswordSink_h__ */
