/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIMIMEService.idl
 */

#ifndef __gen_nsIMIMEService_h__
#define __gen_nsIMIMEService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsIMIMEInfo_h__
#include "nsIMIMEInfo.h"
#endif

#ifndef __gen_nsIURI_h__
#include "nsIURI.h"
#endif

#ifndef __gen_nsIFile_h__
#include "nsIFile.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
#define NS_MIMESERVICE_CID                           \
{ /* 03af31da-3109-11d3-8cd0-0060b0fc14a3 */         \
    0x03af31da,                                      \
    0x3109,                                          \
    0x11d3,                                          \
    {0x8c, 0xd0, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3} \
}

/* starting interface:    nsIMIMEService */
#define NS_IMIMESERVICE_IID_STR "6c424c90-2fe7-11d3-a164-0050041caf44"

#define NS_IMIMESERVICE_IID \
  {0x6c424c90, 0x2fe7, 0x11d3, \
    { 0xa1, 0x64, 0x00, 0x50, 0x04, 0x1c, 0xaf, 0x44 }}

class NS_NO_VTABLE nsIMIMEService : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMIMESERVICE_IID)

  /* nsIMIMEInfo GetFromExtension (in string aFileExt); */
  NS_IMETHOD GetFromExtension(const char *aFileExt, nsIMIMEInfo **_retval) = 0;

  /* nsIMIMEInfo GetFromMIMEType (in string aMIMEType); */
  NS_IMETHOD GetFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval) = 0;

  /* string GetTypeFromExtension (in string aFileExt); */
  NS_IMETHOD GetTypeFromExtension(const char *aFileExt, char **_retval) = 0;

  /* string GetTypeFromURI (in nsIURI aURI); */
  NS_IMETHOD GetTypeFromURI(nsIURI *aURI, char **_retval) = 0;

  /* string GetTypeFromFile (in nsIFile aFile); */
  NS_IMETHOD GetTypeFromFile(nsIFile *aFile, char **_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMIMESERVICE \
  NS_IMETHOD GetFromExtension(const char *aFileExt, nsIMIMEInfo **_retval); \
  NS_IMETHOD GetFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval); \
  NS_IMETHOD GetTypeFromExtension(const char *aFileExt, char **_retval); \
  NS_IMETHOD GetTypeFromURI(nsIURI *aURI, char **_retval); \
  NS_IMETHOD GetTypeFromFile(nsIFile *aFile, char **_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMIMESERVICE(_to) \
  NS_IMETHOD GetFromExtension(const char *aFileExt, nsIMIMEInfo **_retval) { return _to GetFromExtension(aFileExt, _retval); } \
  NS_IMETHOD GetFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval) { return _to GetFromMIMEType(aMIMEType, _retval); } \
  NS_IMETHOD GetTypeFromExtension(const char *aFileExt, char **_retval) { return _to GetTypeFromExtension(aFileExt, _retval); } \
  NS_IMETHOD GetTypeFromURI(nsIURI *aURI, char **_retval) { return _to GetTypeFromURI(aURI, _retval); } \
  NS_IMETHOD GetTypeFromFile(nsIFile *aFile, char **_retval) { return _to GetTypeFromFile(aFile, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIMIMESERVICE(_to) \
  NS_IMETHOD GetFromExtension(const char *aFileExt, nsIMIMEInfo **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetFromExtension(aFileExt, _retval); } \
  NS_IMETHOD GetFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetFromMIMEType(aMIMEType, _retval); } \
  NS_IMETHOD GetTypeFromExtension(const char *aFileExt, char **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetTypeFromExtension(aFileExt, _retval); } \
  NS_IMETHOD GetTypeFromURI(nsIURI *aURI, char **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetTypeFromURI(aURI, _retval); } \
  NS_IMETHOD GetTypeFromFile(nsIFile *aFile, char **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetTypeFromFile(aFile, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMIMEService : public nsIMIMEService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMIMESERVICE

  nsMIMEService();
  virtual ~nsMIMEService();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsMIMEService, nsIMIMEService)

nsMIMEService::nsMIMEService()
{
  /* member initializers and constructor code */
}

nsMIMEService::~nsMIMEService()
{
  /* destructor code */
}

/* nsIMIMEInfo GetFromExtension (in string aFileExt); */
NS_IMETHODIMP nsMIMEService::GetFromExtension(const char *aFileExt, nsIMIMEInfo **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIMIMEInfo GetFromMIMEType (in string aMIMEType); */
NS_IMETHODIMP nsMIMEService::GetFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* string GetTypeFromExtension (in string aFileExt); */
NS_IMETHODIMP nsMIMEService::GetTypeFromExtension(const char *aFileExt, char **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* string GetTypeFromURI (in nsIURI aURI); */
NS_IMETHODIMP nsMIMEService::GetTypeFromURI(nsIURI *aURI, char **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* string GetTypeFromFile (in nsIFile aFile); */
NS_IMETHODIMP nsMIMEService::GetTypeFromFile(nsIFile *aFile, char **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMIMEService_h__ */
