/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM inIPNGEncoder.idl
 */

#ifndef __gen_inIPNGEncoder_h__
#define __gen_inIPNGEncoder_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class inIBitmap; /* forward declaration */


/* starting interface:    inIPNGEncoder */
#define INIPNGENCODER_IID_STR "ec4ce7fc-4268-4f6b-868a-6c0fe3863212"

#define INIPNGENCODER_IID \
  {0xec4ce7fc, 0x4268, 0x4f6b, \
    { 0x86, 0x8a, 0x6c, 0x0f, 0xe3, 0x86, 0x32, 0x12 }}

class NS_NO_VTABLE inIPNGEncoder : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(INIPNGENCODER_IID)

  enum { BIT8 = 0 };

  enum { BIT24 = 1 };

  /* void writePNG (in inIBitmap aBitmap, in wstring aURL, in short aType); */
  NS_IMETHOD WritePNG(inIBitmap *aBitmap, const PRUnichar *aURL, PRInt16 aType) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_INIPNGENCODER \
  NS_IMETHOD WritePNG(inIBitmap *aBitmap, const PRUnichar *aURL, PRInt16 aType); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_INIPNGENCODER(_to) \
  NS_IMETHOD WritePNG(inIBitmap *aBitmap, const PRUnichar *aURL, PRInt16 aType) { return _to WritePNG(aBitmap, aURL, aType); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_INIPNGENCODER(_to) \
  NS_IMETHOD WritePNG(inIBitmap *aBitmap, const PRUnichar *aURL, PRInt16 aType) { return !_to ? NS_ERROR_NULL_POINTER : _to->WritePNG(aBitmap, aURL, aType); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class inPNGEncoder : public inIPNGEncoder
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_INIPNGENCODER

  inPNGEncoder();
  virtual ~inPNGEncoder();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(inPNGEncoder, inIPNGEncoder)

inPNGEncoder::inPNGEncoder()
{
  /* member initializers and constructor code */
}

inPNGEncoder::~inPNGEncoder()
{
  /* destructor code */
}

/* void writePNG (in inIBitmap aBitmap, in wstring aURL, in short aType); */
NS_IMETHODIMP inPNGEncoder::WritePNG(inIBitmap *aBitmap, const PRUnichar *aURL, PRInt16 aType)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_inIPNGEncoder_h__ */
