/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is mozilla.org code.
 * 
 * The Initial Developer of the Original Code is Christopher Blizzard.
 * Portions created Christopher Blizzard are Copyright (C) Christopher
 * Blizzard.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "nsISupports.idl"

[scriptable, uuid(0bafc924-1dd2-11b2-8345-b59762ae0df7)]
interface nsIXRemoteClient : nsISupports
{
  /**
   * Initializes the client
   */
   void init();

  /**
   * Sends a command to a running instance.  If it returns false then
   * there is no running instance.
  */
  boolean sendCommand(in string aCommand);

  /**
   * Shuts down the client
   */
  void shutdown();
   
};
