/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsINetModuleMgr.idl
 */

#ifndef __gen_nsINetModuleMgr_h__
#define __gen_nsINetModuleMgr_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsISimpleEnumerator_h__
#include "nsISimpleEnumerator.h"
#endif

#ifndef __gen_nsINetNotify_h__
#include "nsINetNotify.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIEventQueue; /* forward declaration */

// {4EBDAFE0-13BA-11d3-A15A-0050041CAF44}
#define NS_NETMODULEMGR_CID \
{ 0x4ebdafe0, 0x13ba, 0x11d3, { 0xa1, 0x5a, 0x0, 0x50, 0x4, 0x1c, 0xaf, 0x44 } }
// The list of available CONTRACTIDS to register for notification on.
#define NS_NETWORK_MODULE_MANAGER_HTTP_REQUEST_CONTRACTID          "@mozilla.org/network/moduleMgr/http/request;1"
#define NS_NETWORK_MODULE_MANAGER_HTTP_RESPONSE_CONTRACTID         "@mozilla.org/network/moduleMgr/http/response;1"

/* starting interface:    nsINetModuleMgr */
#define NS_INETMODULEMGR_IID_STR "ff9ead40-0ef2-11d3-9de6-0010a4053fd0"

#define NS_INETMODULEMGR_IID \
  {0xff9ead40, 0x0ef2, 0x11d3, \
    { 0x9d, 0xe6, 0x00, 0x10, 0xa4, 0x05, 0x3f, 0xd0 }}

class NS_NO_VTABLE nsINetModuleMgr : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_INETMODULEMGR_IID)

  /* void registerModule (in string aTopic, in nsINetNotify aNotify); */
  NS_IMETHOD RegisterModule(const char *aTopic, nsINetNotify *aNotify) = 0;

  /* void unregisterModule (in string aTopic, in nsINetNotify aNotify); */
  NS_IMETHOD UnregisterModule(const char *aTopic, nsINetNotify *aNotify) = 0;

  /* void enumerateModules (in string aTopic, out nsISimpleEnumerator aEnumerator); */
  NS_IMETHOD EnumerateModules(const char *aTopic, nsISimpleEnumerator **aEnumerator) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSINETMODULEMGR \
  NS_IMETHOD RegisterModule(const char *aTopic, nsINetNotify *aNotify); \
  NS_IMETHOD UnregisterModule(const char *aTopic, nsINetNotify *aNotify); \
  NS_IMETHOD EnumerateModules(const char *aTopic, nsISimpleEnumerator **aEnumerator); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSINETMODULEMGR(_to) \
  NS_IMETHOD RegisterModule(const char *aTopic, nsINetNotify *aNotify) { return _to RegisterModule(aTopic, aNotify); } \
  NS_IMETHOD UnregisterModule(const char *aTopic, nsINetNotify *aNotify) { return _to UnregisterModule(aTopic, aNotify); } \
  NS_IMETHOD EnumerateModules(const char *aTopic, nsISimpleEnumerator **aEnumerator) { return _to EnumerateModules(aTopic, aEnumerator); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSINETMODULEMGR(_to) \
  NS_IMETHOD RegisterModule(const char *aTopic, nsINetNotify *aNotify) { return !_to ? NS_ERROR_NULL_POINTER : _to->RegisterModule(aTopic, aNotify); } \
  NS_IMETHOD UnregisterModule(const char *aTopic, nsINetNotify *aNotify) { return !_to ? NS_ERROR_NULL_POINTER : _to->UnregisterModule(aTopic, aNotify); } \
  NS_IMETHOD EnumerateModules(const char *aTopic, nsISimpleEnumerator **aEnumerator) { return !_to ? NS_ERROR_NULL_POINTER : _to->EnumerateModules(aTopic, aEnumerator); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsNetModuleMgr : public nsINetModuleMgr
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSINETMODULEMGR

  nsNetModuleMgr();
  virtual ~nsNetModuleMgr();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsNetModuleMgr, nsINetModuleMgr)

nsNetModuleMgr::nsNetModuleMgr()
{
  /* member initializers and constructor code */
}

nsNetModuleMgr::~nsNetModuleMgr()
{
  /* destructor code */
}

/* void registerModule (in string aTopic, in nsINetNotify aNotify); */
NS_IMETHODIMP nsNetModuleMgr::RegisterModule(const char *aTopic, nsINetNotify *aNotify)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void unregisterModule (in string aTopic, in nsINetNotify aNotify); */
NS_IMETHODIMP nsNetModuleMgr::UnregisterModule(const char *aTopic, nsINetNotify *aNotify)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void enumerateModules (in string aTopic, out nsISimpleEnumerator aEnumerator); */
NS_IMETHODIMP nsNetModuleMgr::EnumerateModules(const char *aTopic, nsISimpleEnumerator **aEnumerator)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsINetModuleMgr_h__ */
