/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIMsgMailView.idl
 */

#ifndef __gen_nsIMsgMailView_h__
#define __gen_nsIMsgMailView_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsISupportsArray_h__
#include "nsISupportsArray.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIMsgSearchTerm; /* forward declaration */


/* starting interface:    nsIMsgMailView */
#define NS_IMSGMAILVIEW_IID_STR "82496947-84f9-42ac-87d1-88b0ccaf114b"

#define NS_IMSGMAILVIEW_IID \
  {0x82496947, 0x84f9, 0x42ac, \
    { 0x87, 0xd1, 0x88, 0xb0, 0xcc, 0xaf, 0x11, 0x4b }}

class NS_NO_VTABLE nsIMsgMailView : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMSGMAILVIEW_IID)

  /* attribute wstring mailViewName; */
  NS_IMETHOD GetMailViewName(PRUnichar * *aMailViewName) = 0;
  NS_IMETHOD SetMailViewName(const PRUnichar * aMailViewName) = 0;

  /* attribute nsISupportsArray searchTerms; */
  NS_IMETHOD GetSearchTerms(nsISupportsArray * *aSearchTerms) = 0;
  NS_IMETHOD SetSearchTerms(nsISupportsArray * aSearchTerms) = 0;

  /* void appendTerm (in nsIMsgSearchTerm term); */
  NS_IMETHOD AppendTerm(nsIMsgSearchTerm *term) = 0;

  /* nsIMsgSearchTerm createTerm (); */
  NS_IMETHOD CreateTerm(nsIMsgSearchTerm **_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMSGMAILVIEW \
  NS_IMETHOD GetMailViewName(PRUnichar * *aMailViewName); \
  NS_IMETHOD SetMailViewName(const PRUnichar * aMailViewName); \
  NS_IMETHOD GetSearchTerms(nsISupportsArray * *aSearchTerms); \
  NS_IMETHOD SetSearchTerms(nsISupportsArray * aSearchTerms); \
  NS_IMETHOD AppendTerm(nsIMsgSearchTerm *term); \
  NS_IMETHOD CreateTerm(nsIMsgSearchTerm **_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMSGMAILVIEW(_to) \
  NS_IMETHOD GetMailViewName(PRUnichar * *aMailViewName) { return _to GetMailViewName(aMailViewName); } \
  NS_IMETHOD SetMailViewName(const PRUnichar * aMailViewName) { return _to SetMailViewName(aMailViewName); } \
  NS_IMETHOD GetSearchTerms(nsISupportsArray * *aSearchTerms) { return _to GetSearchTerms(aSearchTerms); } \
  NS_IMETHOD SetSearchTerms(nsISupportsArray * aSearchTerms) { return _to SetSearchTerms(aSearchTerms); } \
  NS_IMETHOD AppendTerm(nsIMsgSearchTerm *term) { return _to AppendTerm(term); } \
  NS_IMETHOD CreateTerm(nsIMsgSearchTerm **_retval) { return _to CreateTerm(_retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIMSGMAILVIEW(_to) \
  NS_IMETHOD GetMailViewName(PRUnichar * *aMailViewName) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetMailViewName(aMailViewName); } \
  NS_IMETHOD SetMailViewName(const PRUnichar * aMailViewName) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetMailViewName(aMailViewName); } \
  NS_IMETHOD GetSearchTerms(nsISupportsArray * *aSearchTerms) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSearchTerms(aSearchTerms); } \
  NS_IMETHOD SetSearchTerms(nsISupportsArray * aSearchTerms) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetSearchTerms(aSearchTerms); } \
  NS_IMETHOD AppendTerm(nsIMsgSearchTerm *term) { return !_to ? NS_ERROR_NULL_POINTER : _to->AppendTerm(term); } \
  NS_IMETHOD CreateTerm(nsIMsgSearchTerm **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->CreateTerm(_retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMsgMailView : public nsIMsgMailView
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMSGMAILVIEW

  nsMsgMailView();
  virtual ~nsMsgMailView();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsMsgMailView, nsIMsgMailView)

nsMsgMailView::nsMsgMailView()
{
  /* member initializers and constructor code */
}

nsMsgMailView::~nsMsgMailView()
{
  /* destructor code */
}

/* attribute wstring mailViewName; */
NS_IMETHODIMP nsMsgMailView::GetMailViewName(PRUnichar * *aMailViewName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsMsgMailView::SetMailViewName(const PRUnichar * aMailViewName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute nsISupportsArray searchTerms; */
NS_IMETHODIMP nsMsgMailView::GetSearchTerms(nsISupportsArray * *aSearchTerms)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsMsgMailView::SetSearchTerms(nsISupportsArray * aSearchTerms)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void appendTerm (in nsIMsgSearchTerm term); */
NS_IMETHODIMP nsMsgMailView::AppendTerm(nsIMsgSearchTerm *term)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIMsgSearchTerm createTerm (); */
NS_IMETHODIMP nsMsgMailView::CreateTerm(nsIMsgSearchTerm **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMsgMailView_h__ */
