/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ipcILockNotify.idl
 */

#ifndef __gen_ipcILockNotify_h__
#define __gen_ipcILockNotify_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    ipcILockNotify */
#define IPCILOCKNOTIFY_IID_STR "b4ac1160-98a4-4030-aecc-01137a05c94b"

#define IPCILOCKNOTIFY_IID \
  {0xb4ac1160, 0x98a4, 0x4030, \
    { 0xae, 0xcc, 0x01, 0x13, 0x7a, 0x05, 0xc9, 0x4b }}

class NS_NO_VTABLE ipcILockNotify : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(IPCILOCKNOTIFY_IID)

  /**
     * Called to complete a call to AcquireLock.
     *
     * @status aLockName
     *         specifies the name of the lock
     * @status aStatus
     *         NS_OK if lock has been acquired; reason for failure otherwise.
     */
  /* void onAcquireLockComplete (in string aLockName, in nsresult aStatus); */
  NS_IMETHOD OnAcquireLockComplete(const char *aLockName, nsresult aStatus) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_IPCILOCKNOTIFY \
  NS_IMETHOD OnAcquireLockComplete(const char *aLockName, nsresult aStatus); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_IPCILOCKNOTIFY(_to) \
  NS_IMETHOD OnAcquireLockComplete(const char *aLockName, nsresult aStatus) { return _to OnAcquireLockComplete(aLockName, aStatus); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_IPCILOCKNOTIFY(_to) \
  NS_IMETHOD OnAcquireLockComplete(const char *aLockName, nsresult aStatus) { return !_to ? NS_ERROR_NULL_POINTER : _to->OnAcquireLockComplete(aLockName, aStatus); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class _MYCLASS_ : public ipcILockNotify
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_IPCILOCKNOTIFY

  _MYCLASS_();
  virtual ~_MYCLASS_();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(_MYCLASS_, ipcILockNotify)

_MYCLASS_::_MYCLASS_()
{
  /* member initializers and constructor code */
}

_MYCLASS_::~_MYCLASS_()
{
  /* destructor code */
}

/* void onAcquireLockComplete (in string aLockName, in nsresult aStatus); */
NS_IMETHODIMP _MYCLASS_::OnAcquireLockComplete(const char *aLockName, nsresult aStatus)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_ipcILockNotify_h__ */
