/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsISOCKS4SocketProvider.idl
 */

#ifndef __gen_nsISOCKS4SocketProvider_h__
#define __gen_nsISOCKS4SocketProvider_h__


#ifndef __gen_nsISocketProvider_h__
#include "nsISocketProvider.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsISOCKS4SocketProvider */
#define NS_ISOCKS4SOCKETPROVIDER_IID_STR "f7c9f5f4-4451-41c3-a28a-5ba2447fbace"

#define NS_ISOCKS4SOCKETPROVIDER_IID \
  {0xf7c9f5f4, 0x4451, 0x41c3, \
    { 0xa2, 0x8a, 0x5b, 0xa2, 0x44, 0x7f, 0xba, 0xce }}

class NS_NO_VTABLE nsISOCKS4SocketProvider : public nsISocketProvider {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ISOCKS4SOCKETPROVIDER_IID)

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSISOCKS4SOCKETPROVIDER \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSISOCKS4SOCKETPROVIDER(_to) \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSISOCKS4SOCKETPROVIDER(_to) \
  /* no methods! */

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsSOCKS4SocketProvider : public nsISOCKS4SocketProvider
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSISOCKS4SOCKETPROVIDER

  nsSOCKS4SocketProvider();
  virtual ~nsSOCKS4SocketProvider();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsSOCKS4SocketProvider, nsISOCKS4SocketProvider)

nsSOCKS4SocketProvider::nsSOCKS4SocketProvider()
{
  /* member initializers and constructor code */
}

nsSOCKS4SocketProvider::~nsSOCKS4SocketProvider()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif

#define NS_ISOCKS4SOCKETPROVIDER_CONTRACTID		NS_NETWORK_SOCKET_CONTRACTID_PREFIX "socks4"
#define NS_ISOCKS4SOCKETPROVIDER_CLASSNAME	"Mozilla SOCKS4 Socket Provider Component"

#endif /* __gen_nsISOCKS4SocketProvider_h__ */
