/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsITreeContentView.idl
 */

#ifndef __gen_nsITreeContentView_h__
#define __gen_nsITreeContentView_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsIDOMElement_h__
#include "nsIDOMElement.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsITreeContentView */
#define NS_ITREECONTENTVIEW_IID_STR "972ae9ff-c7b3-4b60-a51e-a20a9de36947"

#define NS_ITREECONTENTVIEW_IID \
  {0x972ae9ff, 0xc7b3, 0x4b60, \
    { 0xa5, 0x1e, 0xa2, 0x0a, 0x9d, 0xe3, 0x69, 0x47 }}

class NS_NO_VTABLE nsITreeContentView : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ITREECONTENTVIEW_IID)

  /**
   * The element in the DOM which this view uses as root content.
   */
  /* readonly attribute nsIDOMElement root; */
  NS_IMETHOD GetRoot(nsIDOMElement * *aRoot) = 0;

  /**
   * Retrieve the content item associated with the specified index.
   */
  /* nsIDOMElement getItemAtIndex (in long index); */
  NS_IMETHOD GetItemAtIndex(PRInt32 index, nsIDOMElement **_retval) = 0;

  /**
   * Retrieve the index associated with the specified content item.
   */
  /* long getIndexOfItem (in nsIDOMElement item); */
  NS_IMETHOD GetIndexOfItem(nsIDOMElement *item, PRInt32 *_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSITREECONTENTVIEW \
  NS_IMETHOD GetRoot(nsIDOMElement * *aRoot); \
  NS_IMETHOD GetItemAtIndex(PRInt32 index, nsIDOMElement **_retval); \
  NS_IMETHOD GetIndexOfItem(nsIDOMElement *item, PRInt32 *_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSITREECONTENTVIEW(_to) \
  NS_IMETHOD GetRoot(nsIDOMElement * *aRoot) { return _to GetRoot(aRoot); } \
  NS_IMETHOD GetItemAtIndex(PRInt32 index, nsIDOMElement **_retval) { return _to GetItemAtIndex(index, _retval); } \
  NS_IMETHOD GetIndexOfItem(nsIDOMElement *item, PRInt32 *_retval) { return _to GetIndexOfItem(item, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSITREECONTENTVIEW(_to) \
  NS_IMETHOD GetRoot(nsIDOMElement * *aRoot) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetRoot(aRoot); } \
  NS_IMETHOD GetItemAtIndex(PRInt32 index, nsIDOMElement **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetItemAtIndex(index, _retval); } \
  NS_IMETHOD GetIndexOfItem(nsIDOMElement *item, PRInt32 *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetIndexOfItem(item, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsTreeContentView : public nsITreeContentView
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSITREECONTENTVIEW

  nsTreeContentView();
  virtual ~nsTreeContentView();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsTreeContentView, nsITreeContentView)

nsTreeContentView::nsTreeContentView()
{
  /* member initializers and constructor code */
}

nsTreeContentView::~nsTreeContentView()
{
  /* destructor code */
}

/* readonly attribute nsIDOMElement root; */
NS_IMETHODIMP nsTreeContentView::GetRoot(nsIDOMElement * *aRoot)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIDOMElement getItemAtIndex (in long index); */
NS_IMETHODIMP nsTreeContentView::GetItemAtIndex(PRInt32 index, nsIDOMElement **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* long getIndexOfItem (in nsIDOMElement item); */
NS_IMETHODIMP nsTreeContentView::GetIndexOfItem(nsIDOMElement *item, PRInt32 *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsITreeContentView_h__ */
