/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Boris Zbarsky
 * <bzbarsky@mit.edu> Portions created by Boris Zbarsky are Copyright
 * (C) 2001.  All Rights Reserved.
 *
 * Contributor(s):
 */

#include "nsISupports.idl"

%{ C++
#include "nsString.h" // needed for AString -> nsAString, unfortunately
%}

interface nsILineInputStream;

[scriptable, uuid(e7f17108-1dd1-11b2-8b9a-fda151ea0240)]
interface nsILineInputStream : nsISupports
{
  /**
   * Read a single line from the stream
   * Return false for end of file, true otherwise
   */
  boolean readLine(out AString aLine);
};
