/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIExpatSink.idl
 */

#ifndef __gen_nsIExpatSink_h__
#define __gen_nsIExpatSink_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIExpatSink */
#define NS_IEXPATSINK_IID_STR "1deea160-c661-11d5-84cc-0010a4e0c706"

#define NS_IEXPATSINK_IID \
  {0x1deea160, 0xc661, 0x11d5, \
    { 0x84, 0xcc, 0x00, 0x10, 0xa4, 0xe0, 0xc7, 0x06 }}

class NS_NO_VTABLE nsIExpatSink : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IEXPATSINK_IID)

  /* void HandleStartElement (in wstring aName, [array, size_is (aAttsCount)] in wstring aAtts, in unsigned long aAttsCount, in unsigned long aIndex, in unsigned long aLineNumber); */
  NS_IMETHOD HandleStartElement(const PRUnichar *aName, const PRUnichar **aAtts, PRUint32 aAttsCount, PRUint32 aIndex, PRUint32 aLineNumber) = 0;

  /* void HandleEndElement (in wstring aName); */
  NS_IMETHOD HandleEndElement(const PRUnichar *aName) = 0;

  /* void HandleComment (in wstring aName); */
  NS_IMETHOD HandleComment(const PRUnichar *aName) = 0;

  /* void HandleCDataSection (in wstring aData, in unsigned long aLength); */
  NS_IMETHOD HandleCDataSection(const PRUnichar *aData, PRUint32 aLength) = 0;

  /* void HandleDoctypeDecl (in AString aSubset, in AString aName, in AString aSystemId, in AString aPublicId, in nsISupports aCatalogData); */
  NS_IMETHOD HandleDoctypeDecl(const nsAString & aSubset, const nsAString & aName, const nsAString & aSystemId, const nsAString & aPublicId, nsISupports *aCatalogData) = 0;

  /* void HandleCharacterData (in wstring aData, in unsigned long aLength); */
  NS_IMETHOD HandleCharacterData(const PRUnichar *aData, PRUint32 aLength) = 0;

  /* void HandleProcessingInstruction (in wstring aTarget, in wstring aData); */
  NS_IMETHOD HandleProcessingInstruction(const PRUnichar *aTarget, const PRUnichar *aData) = 0;

  /**
   * Handle the XML Declaration.
   *
   * @param aData    The string.
   * @param aLength  The length of the declaration from
   *                 opening '<' to closing '>'.
   **/
  /* void HandleXMLDeclaration (in wstring aData, in unsigned long aLength); */
  NS_IMETHOD HandleXMLDeclaration(const PRUnichar *aData, PRUint32 aLength) = 0;

  /* void ReportError (in wstring aErrorText, in wstring aSourceText); */
  NS_IMETHOD ReportError(const PRUnichar *aErrorText, const PRUnichar *aSourceText) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIEXPATSINK \
  NS_IMETHOD HandleStartElement(const PRUnichar *aName, const PRUnichar **aAtts, PRUint32 aAttsCount, PRUint32 aIndex, PRUint32 aLineNumber); \
  NS_IMETHOD HandleEndElement(const PRUnichar *aName); \
  NS_IMETHOD HandleComment(const PRUnichar *aName); \
  NS_IMETHOD HandleCDataSection(const PRUnichar *aData, PRUint32 aLength); \
  NS_IMETHOD HandleDoctypeDecl(const nsAString & aSubset, const nsAString & aName, const nsAString & aSystemId, const nsAString & aPublicId, nsISupports *aCatalogData); \
  NS_IMETHOD HandleCharacterData(const PRUnichar *aData, PRUint32 aLength); \
  NS_IMETHOD HandleProcessingInstruction(const PRUnichar *aTarget, const PRUnichar *aData); \
  NS_IMETHOD HandleXMLDeclaration(const PRUnichar *aData, PRUint32 aLength); \
  NS_IMETHOD ReportError(const PRUnichar *aErrorText, const PRUnichar *aSourceText); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIEXPATSINK(_to) \
  NS_IMETHOD HandleStartElement(const PRUnichar *aName, const PRUnichar **aAtts, PRUint32 aAttsCount, PRUint32 aIndex, PRUint32 aLineNumber) { return _to HandleStartElement(aName, aAtts, aAttsCount, aIndex, aLineNumber); } \
  NS_IMETHOD HandleEndElement(const PRUnichar *aName) { return _to HandleEndElement(aName); } \
  NS_IMETHOD HandleComment(const PRUnichar *aName) { return _to HandleComment(aName); } \
  NS_IMETHOD HandleCDataSection(const PRUnichar *aData, PRUint32 aLength) { return _to HandleCDataSection(aData, aLength); } \
  NS_IMETHOD HandleDoctypeDecl(const nsAString & aSubset, const nsAString & aName, const nsAString & aSystemId, const nsAString & aPublicId, nsISupports *aCatalogData) { return _to HandleDoctypeDecl(aSubset, aName, aSystemId, aPublicId, aCatalogData); } \
  NS_IMETHOD HandleCharacterData(const PRUnichar *aData, PRUint32 aLength) { return _to HandleCharacterData(aData, aLength); } \
  NS_IMETHOD HandleProcessingInstruction(const PRUnichar *aTarget, const PRUnichar *aData) { return _to HandleProcessingInstruction(aTarget, aData); } \
  NS_IMETHOD HandleXMLDeclaration(const PRUnichar *aData, PRUint32 aLength) { return _to HandleXMLDeclaration(aData, aLength); } \
  NS_IMETHOD ReportError(const PRUnichar *aErrorText, const PRUnichar *aSourceText) { return _to ReportError(aErrorText, aSourceText); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIEXPATSINK(_to) \
  NS_IMETHOD HandleStartElement(const PRUnichar *aName, const PRUnichar **aAtts, PRUint32 aAttsCount, PRUint32 aIndex, PRUint32 aLineNumber) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleStartElement(aName, aAtts, aAttsCount, aIndex, aLineNumber); } \
  NS_IMETHOD HandleEndElement(const PRUnichar *aName) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleEndElement(aName); } \
  NS_IMETHOD HandleComment(const PRUnichar *aName) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleComment(aName); } \
  NS_IMETHOD HandleCDataSection(const PRUnichar *aData, PRUint32 aLength) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleCDataSection(aData, aLength); } \
  NS_IMETHOD HandleDoctypeDecl(const nsAString & aSubset, const nsAString & aName, const nsAString & aSystemId, const nsAString & aPublicId, nsISupports *aCatalogData) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleDoctypeDecl(aSubset, aName, aSystemId, aPublicId, aCatalogData); } \
  NS_IMETHOD HandleCharacterData(const PRUnichar *aData, PRUint32 aLength) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleCharacterData(aData, aLength); } \
  NS_IMETHOD HandleProcessingInstruction(const PRUnichar *aTarget, const PRUnichar *aData) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleProcessingInstruction(aTarget, aData); } \
  NS_IMETHOD HandleXMLDeclaration(const PRUnichar *aData, PRUint32 aLength) { return !_to ? NS_ERROR_NULL_POINTER : _to->HandleXMLDeclaration(aData, aLength); } \
  NS_IMETHOD ReportError(const PRUnichar *aErrorText, const PRUnichar *aSourceText) { return !_to ? NS_ERROR_NULL_POINTER : _to->ReportError(aErrorText, aSourceText); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsExpatSink : public nsIExpatSink
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIEXPATSINK

  nsExpatSink();
  virtual ~nsExpatSink();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsExpatSink, nsIExpatSink)

nsExpatSink::nsExpatSink()
{
  /* member initializers and constructor code */
}

nsExpatSink::~nsExpatSink()
{
  /* destructor code */
}

/* void HandleStartElement (in wstring aName, [array, size_is (aAttsCount)] in wstring aAtts, in unsigned long aAttsCount, in unsigned long aIndex, in unsigned long aLineNumber); */
NS_IMETHODIMP nsExpatSink::HandleStartElement(const PRUnichar *aName, const PRUnichar **aAtts, PRUint32 aAttsCount, PRUint32 aIndex, PRUint32 aLineNumber)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleEndElement (in wstring aName); */
NS_IMETHODIMP nsExpatSink::HandleEndElement(const PRUnichar *aName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleComment (in wstring aName); */
NS_IMETHODIMP nsExpatSink::HandleComment(const PRUnichar *aName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleCDataSection (in wstring aData, in unsigned long aLength); */
NS_IMETHODIMP nsExpatSink::HandleCDataSection(const PRUnichar *aData, PRUint32 aLength)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleDoctypeDecl (in AString aSubset, in AString aName, in AString aSystemId, in AString aPublicId, in nsISupports aCatalogData); */
NS_IMETHODIMP nsExpatSink::HandleDoctypeDecl(const nsAString & aSubset, const nsAString & aName, const nsAString & aSystemId, const nsAString & aPublicId, nsISupports *aCatalogData)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleCharacterData (in wstring aData, in unsigned long aLength); */
NS_IMETHODIMP nsExpatSink::HandleCharacterData(const PRUnichar *aData, PRUint32 aLength)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleProcessingInstruction (in wstring aTarget, in wstring aData); */
NS_IMETHODIMP nsExpatSink::HandleProcessingInstruction(const PRUnichar *aTarget, const PRUnichar *aData)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void HandleXMLDeclaration (in wstring aData, in unsigned long aLength); */
NS_IMETHODIMP nsExpatSink::HandleXMLDeclaration(const PRUnichar *aData, PRUint32 aLength)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void ReportError (in wstring aErrorText, in wstring aSourceText); */
NS_IMETHODIMP nsExpatSink::ReportError(const PRUnichar *aErrorText, const PRUnichar *aSourceText)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIExpatSink_h__ */
