/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsICookieConsent.idl
 */

#ifndef __gen_nsICookieConsent_h__
#define __gen_nsICookieConsent_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIHttpChannel; /* forward declaration */


/* starting interface:    nsICookieConsent */
#define NS_ICOOKIECONSENT_IID_STR "f5a34f50-1f39-11d6-a627-0010a401eb10"

#define NS_ICOOKIECONSENT_IID \
  {0xf5a34f50, 0x1f39, 0x11d6, \
    { 0xa6, 0x27, 0x00, 0x10, 0xa4, 0x01, 0xeb, 0x10 }}

class NS_NO_VTABLE nsICookieConsent : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ICOOKIECONSENT_IID)

  /* long getConsent (in string aURI, in nsIHttpChannel aHttpChannel); */
  NS_IMETHOD GetConsent(const char *aURI, nsIHttpChannel *aHttpChannel, PRInt32 *_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSICOOKIECONSENT \
  NS_IMETHOD GetConsent(const char *aURI, nsIHttpChannel *aHttpChannel, PRInt32 *_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSICOOKIECONSENT(_to) \
  NS_IMETHOD GetConsent(const char *aURI, nsIHttpChannel *aHttpChannel, PRInt32 *_retval) { return _to GetConsent(aURI, aHttpChannel, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSICOOKIECONSENT(_to) \
  NS_IMETHOD GetConsent(const char *aURI, nsIHttpChannel *aHttpChannel, PRInt32 *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetConsent(aURI, aHttpChannel, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsCookieConsent : public nsICookieConsent
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSICOOKIECONSENT

  nsCookieConsent();
  virtual ~nsCookieConsent();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsCookieConsent, nsICookieConsent)

nsCookieConsent::nsCookieConsent()
{
  /* member initializers and constructor code */
}

nsCookieConsent::~nsCookieConsent()
{
  /* destructor code */
}

/* long getConsent (in string aURI, in nsIHttpChannel aHttpChannel); */
NS_IMETHODIMP nsCookieConsent::GetConsent(const char *aURI, nsIHttpChannel *aHttpChannel, PRInt32 *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif

#define NS_COOKIECONSENT_CONTRACTID "@mozilla.org/cookie-consent;1"

#endif /* __gen_nsICookieConsent_h__ */
