/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIHttpNotify.idl
 */

#ifndef __gen_nsIHttpNotify_h__
#define __gen_nsIHttpNotify_h__


#ifndef __gen_nsINetNotify_h__
#include "nsINetNotify.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIHttpChannel; /* forward declaration */


/* starting interface:    nsIHttpNotify */
#define NS_IHTTPNOTIFY_IID_STR "5a6dec35-fb37-4460-85bb-1c572521d0cf"

#define NS_IHTTPNOTIFY_IID \
  {0x5a6dec35, 0xfb37, 0x4460, \
    { 0x85, 0xbb, 0x1c, 0x57, 0x25, 0x21, 0xd0, 0xcf }}

class NS_NO_VTABLE nsIHttpNotify : public nsINetNotify {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IHTTPNOTIFY_IID)

  /**
     * Before an HTTP request is sent to the server, this method is called.
     * The implementor of this method can then chose to set any additional
     * headers for this request before the request is actually sent to the
     * server.  The HTTP protocol interpreter blocks on this call until it
     * returns.
     */
  /* void onModifyRequest (in nsIHttpChannel httpChannel); */
  NS_IMETHOD OnModifyRequest(nsIHttpChannel *httpChannel) = 0;

  /**
     * After an HTTP server response is received, this method is called
     * syncronously.  The implementor can interrogate the nsIHttpChannel as
     * it deems necessary.  The HTTP protocol interpreter blocks on this call
     * until it returns.
     */
  /* void onExamineResponse (in nsIHttpChannel httpChannel); */
  NS_IMETHOD OnExamineResponse(nsIHttpChannel *httpChannel) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIHTTPNOTIFY \
  NS_IMETHOD OnModifyRequest(nsIHttpChannel *httpChannel); \
  NS_IMETHOD OnExamineResponse(nsIHttpChannel *httpChannel); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIHTTPNOTIFY(_to) \
  NS_IMETHOD OnModifyRequest(nsIHttpChannel *httpChannel) { return _to OnModifyRequest(httpChannel); } \
  NS_IMETHOD OnExamineResponse(nsIHttpChannel *httpChannel) { return _to OnExamineResponse(httpChannel); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIHTTPNOTIFY(_to) \
  NS_IMETHOD OnModifyRequest(nsIHttpChannel *httpChannel) { return !_to ? NS_ERROR_NULL_POINTER : _to->OnModifyRequest(httpChannel); } \
  NS_IMETHOD OnExamineResponse(nsIHttpChannel *httpChannel) { return !_to ? NS_ERROR_NULL_POINTER : _to->OnExamineResponse(httpChannel); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsHttpNotify : public nsIHttpNotify
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIHTTPNOTIFY

  nsHttpNotify();
  virtual ~nsHttpNotify();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsHttpNotify, nsIHttpNotify)

nsHttpNotify::nsHttpNotify()
{
  /* member initializers and constructor code */
}

nsHttpNotify::~nsHttpNotify()
{
  /* destructor code */
}

/* void onModifyRequest (in nsIHttpChannel httpChannel); */
NS_IMETHODIMP nsHttpNotify::OnModifyRequest(nsIHttpChannel *httpChannel)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void onExamineResponse (in nsIHttpChannel httpChannel); */
NS_IMETHODIMP nsHttpNotify::OnExamineResponse(nsIHttpChannel *httpChannel)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIHttpNotify_h__ */
