/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation. All
 * Rights Reserved.
 * 
 * Original Author: Aaron Leventhal
 * 
 * Contributor(s): 
 * John Gaunt
 */

#include "nsISupports.idl"
#include "nsIAccessible.idl"
#include "domstubs.idl"
#include "nsIAccessibleCaret.idl"

interface nsIDocument;
interface nsIAccessNode;

[scriptable, uuid(8781FC88-355F-4439-881F-6504A0A1CEB6)]
interface nsIAccessibleDocument : nsISupports
{
  readonly attribute AString URL;
  readonly attribute AString title;
  readonly attribute AString mimeType;
  readonly attribute AString docType;
  readonly attribute boolean isEditable;
  AString getNameSpaceURIForID(in short nameSpaceID);
  readonly attribute nsIAccessibleCaret caretAccessible;
  [noscript] readonly attribute voidPtr window;
  [noscript] nsIAccessNode getCachedAccessNode(in voidPtr aUniqueID);
  [noscript] void InvalidateCacheSubtree(in nsIDOMNode aStartNode);
  [noscript] nsIAccessible getAccessibleInParentChain(in nsIDOMNode aDOMNode);
  [noscript] void cacheAccessNode(in voidPtr aUniqueID, in nsIAccessNode aAccessNode);
  [noscript] void destroy();
};
