/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsITimeRecorder.idl
 */

#ifndef __gen_nsITimeRecorder_h__
#define __gen_nsITimeRecorder_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
#define NS_TIMER_STYLE_RESOLUTION 1

/* starting interface:    nsITimeRecorder */
#define NS_ITIMERECORDER_IID_STR "0e5f5cb6-6d29-11d3-aef7-00108300ff91"

#define NS_ITIMERECORDER_IID \
  {0x0e5f5cb6, 0x6d29, 0x11d3, \
    { 0xae, 0xf7, 0x00, 0x10, 0x83, 0x00, 0xff, 0x91 }}

class NS_NO_VTABLE nsITimeRecorder : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ITIMERECORDER_IID)

  /* void EnableTimer (in PRUint32 aTimerID); */
  NS_IMETHOD EnableTimer(PRUint32 aTimerID) = 0;

  /* void DisableTimer (in PRUint32 aTimerID); */
  NS_IMETHOD DisableTimer(PRUint32 aTimerID) = 0;

  /* void IsTimerEnabled (out PRBool aEnabled, in PRUint32 aTimerID); */
  NS_IMETHOD IsTimerEnabled(PRBool *aEnabled, PRUint32 aTimerID) = 0;

  /* void ResetTimer (in PRUint32 aTimerID); */
  NS_IMETHOD ResetTimer(PRUint32 aTimerID) = 0;

  /* void StartTimer (in PRUint32 aTimerID); */
  NS_IMETHOD StartTimer(PRUint32 aTimerID) = 0;

  /* void StopTimer (in PRUint32 aTimerID); */
  NS_IMETHOD StopTimer(PRUint32 aTimerID) = 0;

  /* void PrintTimer (in PRUint32 aTimerID); */
  NS_IMETHOD PrintTimer(PRUint32 aTimerID) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSITIMERECORDER \
  NS_IMETHOD EnableTimer(PRUint32 aTimerID); \
  NS_IMETHOD DisableTimer(PRUint32 aTimerID); \
  NS_IMETHOD IsTimerEnabled(PRBool *aEnabled, PRUint32 aTimerID); \
  NS_IMETHOD ResetTimer(PRUint32 aTimerID); \
  NS_IMETHOD StartTimer(PRUint32 aTimerID); \
  NS_IMETHOD StopTimer(PRUint32 aTimerID); \
  NS_IMETHOD PrintTimer(PRUint32 aTimerID); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSITIMERECORDER(_to) \
  NS_IMETHOD EnableTimer(PRUint32 aTimerID) { return _to EnableTimer(aTimerID); } \
  NS_IMETHOD DisableTimer(PRUint32 aTimerID) { return _to DisableTimer(aTimerID); } \
  NS_IMETHOD IsTimerEnabled(PRBool *aEnabled, PRUint32 aTimerID) { return _to IsTimerEnabled(aEnabled, aTimerID); } \
  NS_IMETHOD ResetTimer(PRUint32 aTimerID) { return _to ResetTimer(aTimerID); } \
  NS_IMETHOD StartTimer(PRUint32 aTimerID) { return _to StartTimer(aTimerID); } \
  NS_IMETHOD StopTimer(PRUint32 aTimerID) { return _to StopTimer(aTimerID); } \
  NS_IMETHOD PrintTimer(PRUint32 aTimerID) { return _to PrintTimer(aTimerID); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSITIMERECORDER(_to) \
  NS_IMETHOD EnableTimer(PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->EnableTimer(aTimerID); } \
  NS_IMETHOD DisableTimer(PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->DisableTimer(aTimerID); } \
  NS_IMETHOD IsTimerEnabled(PRBool *aEnabled, PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->IsTimerEnabled(aEnabled, aTimerID); } \
  NS_IMETHOD ResetTimer(PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->ResetTimer(aTimerID); } \
  NS_IMETHOD StartTimer(PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->StartTimer(aTimerID); } \
  NS_IMETHOD StopTimer(PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->StopTimer(aTimerID); } \
  NS_IMETHOD PrintTimer(PRUint32 aTimerID) { return !_to ? NS_ERROR_NULL_POINTER : _to->PrintTimer(aTimerID); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsTimeRecorder : public nsITimeRecorder
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSITIMERECORDER

  nsTimeRecorder();
  virtual ~nsTimeRecorder();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsTimeRecorder, nsITimeRecorder)

nsTimeRecorder::nsTimeRecorder()
{
  /* member initializers and constructor code */
}

nsTimeRecorder::~nsTimeRecorder()
{
  /* destructor code */
}

/* void EnableTimer (in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::EnableTimer(PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void DisableTimer (in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::DisableTimer(PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void IsTimerEnabled (out PRBool aEnabled, in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::IsTimerEnabled(PRBool *aEnabled, PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void ResetTimer (in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::ResetTimer(PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void StartTimer (in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::StartTimer(PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void StopTimer (in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::StopTimer(PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void PrintTimer (in PRUint32 aTimerID); */
NS_IMETHODIMP nsTimeRecorder::PrintTimer(PRUint32 aTimerID)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsITimeRecorder_h__ */
