/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsINoIncomingServer.idl
 */

#ifndef __gen_nsINoIncomingServer_h__
#define __gen_nsINoIncomingServer_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIFileSpec; /* forward declaration */


/* starting interface:    nsINoIncomingServer */
#define NS_INOINCOMINGSERVER_IID_STR "739b7fb0-5f37-11d3-896b-004005263078"

#define NS_INOINCOMINGSERVER_IID \
  {0x739b7fb0, 0x5f37, 0x11d3, \
    { 0x89, 0x6b, 0x00, 0x40, 0x05, 0x26, 0x30, 0x78 }}

class NS_NO_VTABLE nsINoIncomingServer : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_INOINCOMINGSERVER_IID)

  /* void copyDefaultMessages (in string folderNameOnDisk, in nsIFileSpec parentDir); */
  NS_IMETHOD CopyDefaultMessages(const char *folderNameOnDisk, nsIFileSpec *parentDir) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSINOINCOMINGSERVER \
  NS_IMETHOD CopyDefaultMessages(const char *folderNameOnDisk, nsIFileSpec *parentDir); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSINOINCOMINGSERVER(_to) \
  NS_IMETHOD CopyDefaultMessages(const char *folderNameOnDisk, nsIFileSpec *parentDir) { return _to CopyDefaultMessages(folderNameOnDisk, parentDir); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSINOINCOMINGSERVER(_to) \
  NS_IMETHOD CopyDefaultMessages(const char *folderNameOnDisk, nsIFileSpec *parentDir) { return !_to ? NS_ERROR_NULL_POINTER : _to->CopyDefaultMessages(folderNameOnDisk, parentDir); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsNoIncomingServer : public nsINoIncomingServer
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSINOINCOMINGSERVER

  nsNoIncomingServer();
  virtual ~nsNoIncomingServer();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsNoIncomingServer, nsINoIncomingServer)

nsNoIncomingServer::nsNoIncomingServer()
{
  /* member initializers and constructor code */
}

nsNoIncomingServer::~nsNoIncomingServer()
{
  /* destructor code */
}

/* void copyDefaultMessages (in string folderNameOnDisk, in nsIFileSpec parentDir); */
NS_IMETHODIMP nsNoIncomingServer::CopyDefaultMessages(const char *folderNameOnDisk, nsIFileSpec *parentDir)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsINoIncomingServer_h__ */
