/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIDragTracker.idl
 */

#ifndef __gen_nsIDragTracker_h__
#define __gen_nsIDragTracker_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIDragTracker */
#define NS_IDRAGTRACKER_IID_STR "641efb5c-1dd2-11b2-9067-8020880c3961"

#define NS_IDRAGTRACKER_IID \
  {0x641efb5c, 0x1dd2, 0x11b2, \
    { 0x90, 0x67, 0x80, 0x20, 0x88, 0x0c, 0x39, 0x61 }}

class NS_NO_VTABLE nsIDragTracker : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IDRAGTRACKER_IID)

  /**
    * stopTracking ( )
    *
    * Call to tell the object that is tracking the drag that it
    * can forget about it (mouse left the window or the drop completed).
    */
  /* void stopTracking (); */
  NS_IMETHOD StopTracking(void) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIDRAGTRACKER \
  NS_IMETHOD StopTracking(void); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIDRAGTRACKER(_to) \
  NS_IMETHOD StopTracking(void) { return _to StopTracking(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIDRAGTRACKER(_to) \
  NS_IMETHOD StopTracking(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->StopTracking(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsDragTracker : public nsIDragTracker
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDRAGTRACKER

  nsDragTracker();
  virtual ~nsDragTracker();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsDragTracker, nsIDragTracker)

nsDragTracker::nsDragTracker()
{
  /* member initializers and constructor code */
}

nsDragTracker::~nsDragTracker()
{
  /* destructor code */
}

/* void stopTracking (); */
NS_IMETHODIMP nsDragTracker::StopTracking()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIDragTracker_h__ */
