/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Mozilla Communicator client code,
 * released March 31, 1998.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation.  Portions created by Netscape are
 * Copyright (C) 2000 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *     Douglas Turner <dougt@netscape.com>
 *     Daniel Veditz <dveditz@netscape.com>
 */

#include "nsISupports.idl"

[uuid(eea90d40-b059-11d2-915e-c12b696c9333)]
interface nsIXPIListener : nsISupports
{
    void    onInstallStart(in wstring URL);
    void    onInstallDone(in wstring URL, in long status);
    void    onPackageNameSet(in wstring URL, in wstring UIPackageName, in wstring version);
    void    onItemScheduled(in wstring message );
    void    onFinalizeProgress(in wstring message, in long itemNum, in long totNum );
    void    onLogComment(in wstring comment);
};
