/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * This file contains an interface to the Permission Manager,
 * used for blocking objects (cookies, images etc) from
 * used-defined sites
 *
 * This service broadcasts a PERM_CHANGE_NOTIFICATION notification
 * when any permission changes. You can add yourself as an observer.
 */

#include "nsISupports.idl"
#include "nsISimpleEnumerator.idl"

interface nsIURI;
interface nsIObserver;

[scriptable, uuid(4F6B5E00-0C36-11d5-A535-0010A401EB10)]
interface nsIPermissionManager : nsISupports
{
  /**
   * Allowed return values from the testPermission method
   */
  const PRUint32 UNKNOWN_ACTION = 0;
  const PRUint32 ALLOW_ACTION = 1;
  const PRUint32 DENY_ACTION = 2;

  /**
   * These values are the different types of permissions supported
   */
  const PRUint32 COOKIE_TYPE = 0;
  const PRUint32 IMAGE_TYPE = 1;
  const PRUint32 POPUP_TYPE = 2;

  /**
   * Add permission information for a given URI and permission type.
   * @param uri is the website to add the permission for
   * @param permission is one of the enumerated permission actions above
   * @param type is one of the enumerated permission types above
   */
  void add(in nsIURI uri,
           in PRUint32 type,
           in PRUint32 permission);

  /**
   * Remove permission information for a given URI and permission type.
   * Note that this method takes a host string, not an nsIURI.
   * @param host is the host to remove the permission for
   * @param type is one of the enumerated permission types above
   */
  void remove(in AUTF8String host,
              in PRUint32 type);

  /**
   * Clear permission information for all websites.
   */
  void removeAll();

  /**
   * Test whether a website has permission to perform the given action.
   * @param  uri   the website to be tested
   * @param  type  one of the enumerated types above
   * @return       one of the enumerated permission actions above
   */
  PRUint32 testPermission(in nsIURI uri,
                          in PRUint32 type);

  /**
   * Allows enumeration of all stored permissions
   * @return an nsISimpleEnumerator interface that allows access to
   *         nsIPermission objects
   */
  readonly attribute nsISimpleEnumerator enumerator;
};

%{ C++
#define NS_PERMISSIONMANAGER_CONTRACTID "@mozilla.org/permissionmanager;1"

#define PERM_CHANGE_NOTIFICATION "perm-changed"
%}
