/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM mozISpellI18NUtil.idl
 */

#ifndef __gen_mozISpellI18NUtil_h__
#define __gen_mozISpellI18NUtil_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    mozISpellI18NUtil */
#define MOZISPELLI18NUTIL_IID_STR "24635fe1-2c7f-4d91-9274-b1ac77d93419"

#define MOZISPELLI18NUTIL_IID \
  {0x24635fe1, 0x2c7f, 0x4d91, \
    { 0x92, 0x74, 0xb1, 0xac, 0x77, 0xd9, 0x34, 0x19 }}

/**
  This interface contains various I18N related code used in various places by the spell checker.
*/
class NS_NO_VTABLE mozISpellI18NUtil : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(MOZISPELLI18NUTIL_IID)

  enum { kCheck = 0U };

  enum { kSuggest = 1U };

  /* readonly attribute wstring language; */
  NS_IMETHOD GetLanguage(PRUnichar * *aLanguage) = 0;

  /* attribute wstring charset; */
  NS_IMETHOD GetCharset(PRUnichar * *aCharset) = 0;
  NS_IMETHOD SetCharset(const PRUnichar * aCharset) = 0;

  /* void GetRootForm (in wstring word, in PRUint32 type, [array, size_is (count)] out string words, out PRUint32 count); */
  NS_IMETHOD GetRootForm(const PRUnichar *word, PRUint32 type, char ***words, PRUint32 *count) = 0;

  /* void FromRootForm (in wstring word, [array, size_is (icount)] in string iwords, in PRUint32 icount, [array, size_is (ocount)] out wstring owords, out PRUint32 ocount); */
  NS_IMETHOD FromRootForm(const PRUnichar *word, const char **iwords, PRUint32 icount, PRUnichar ***owords, PRUint32 *ocount) = 0;

  /* void FindNextWord (in wstring word, in PRUint32 length, in PRUint32 offset, out PRInt32 begin, out PRInt32 end); */
  NS_IMETHOD FindNextWord(const PRUnichar *word, PRUint32 length, PRUint32 offset, PRInt32 *begin, PRInt32 *end) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_MOZISPELLI18NUTIL \
  NS_IMETHOD GetLanguage(PRUnichar * *aLanguage); \
  NS_IMETHOD GetCharset(PRUnichar * *aCharset); \
  NS_IMETHOD SetCharset(const PRUnichar * aCharset); \
  NS_IMETHOD GetRootForm(const PRUnichar *word, PRUint32 type, char ***words, PRUint32 *count); \
  NS_IMETHOD FromRootForm(const PRUnichar *word, const char **iwords, PRUint32 icount, PRUnichar ***owords, PRUint32 *ocount); \
  NS_IMETHOD FindNextWord(const PRUnichar *word, PRUint32 length, PRUint32 offset, PRInt32 *begin, PRInt32 *end); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_MOZISPELLI18NUTIL(_to) \
  NS_IMETHOD GetLanguage(PRUnichar * *aLanguage) { return _to GetLanguage(aLanguage); } \
  NS_IMETHOD GetCharset(PRUnichar * *aCharset) { return _to GetCharset(aCharset); } \
  NS_IMETHOD SetCharset(const PRUnichar * aCharset) { return _to SetCharset(aCharset); } \
  NS_IMETHOD GetRootForm(const PRUnichar *word, PRUint32 type, char ***words, PRUint32 *count) { return _to GetRootForm(word, type, words, count); } \
  NS_IMETHOD FromRootForm(const PRUnichar *word, const char **iwords, PRUint32 icount, PRUnichar ***owords, PRUint32 *ocount) { return _to FromRootForm(word, iwords, icount, owords, ocount); } \
  NS_IMETHOD FindNextWord(const PRUnichar *word, PRUint32 length, PRUint32 offset, PRInt32 *begin, PRInt32 *end) { return _to FindNextWord(word, length, offset, begin, end); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_MOZISPELLI18NUTIL(_to) \
  NS_IMETHOD GetLanguage(PRUnichar * *aLanguage) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLanguage(aLanguage); } \
  NS_IMETHOD GetCharset(PRUnichar * *aCharset) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetCharset(aCharset); } \
  NS_IMETHOD SetCharset(const PRUnichar * aCharset) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetCharset(aCharset); } \
  NS_IMETHOD GetRootForm(const PRUnichar *word, PRUint32 type, char ***words, PRUint32 *count) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetRootForm(word, type, words, count); } \
  NS_IMETHOD FromRootForm(const PRUnichar *word, const char **iwords, PRUint32 icount, PRUnichar ***owords, PRUint32 *ocount) { return !_to ? NS_ERROR_NULL_POINTER : _to->FromRootForm(word, iwords, icount, owords, ocount); } \
  NS_IMETHOD FindNextWord(const PRUnichar *word, PRUint32 length, PRUint32 offset, PRInt32 *begin, PRInt32 *end) { return !_to ? NS_ERROR_NULL_POINTER : _to->FindNextWord(word, length, offset, begin, end); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class _MYCLASS_ : public mozISpellI18NUtil
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_MOZISPELLI18NUTIL

  _MYCLASS_();
  virtual ~_MYCLASS_();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(_MYCLASS_, mozISpellI18NUtil)

_MYCLASS_::_MYCLASS_()
{
  /* member initializers and constructor code */
}

_MYCLASS_::~_MYCLASS_()
{
  /* destructor code */
}

/* readonly attribute wstring language; */
NS_IMETHODIMP _MYCLASS_::GetLanguage(PRUnichar * *aLanguage)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute wstring charset; */
NS_IMETHODIMP _MYCLASS_::GetCharset(PRUnichar * *aCharset)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP _MYCLASS_::SetCharset(const PRUnichar * aCharset)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void GetRootForm (in wstring word, in PRUint32 type, [array, size_is (count)] out string words, out PRUint32 count); */
NS_IMETHODIMP _MYCLASS_::GetRootForm(const PRUnichar *word, PRUint32 type, char ***words, PRUint32 *count)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void FromRootForm (in wstring word, [array, size_is (icount)] in string iwords, in PRUint32 icount, [array, size_is (ocount)] out wstring owords, out PRUint32 ocount); */
NS_IMETHODIMP _MYCLASS_::FromRootForm(const PRUnichar *word, const char **iwords, PRUint32 icount, PRUnichar ***owords, PRUint32 *ocount)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void FindNextWord (in wstring word, in PRUint32 length, in PRUint32 offset, out PRInt32 begin, out PRInt32 end); */
NS_IMETHODIMP _MYCLASS_::FindNextWord(const PRUnichar *word, PRUint32 length, PRUint32 offset, PRInt32 *begin, PRInt32 *end)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_mozISpellI18NUtil_h__ */
