/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 2001 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Terry Hayes <thayes@netscape.com>
 */

#include "nsISupports.idl"

interface nsIInterfaceRequestor;

/**
 * nsIClientAuthDialog
 * Provides UI for SSL client-auth dialogs.
 */
[scriptable, uuid(fa4c7520-1433-11d5-ba24-00108303b117)]
interface nsIClientAuthDialogs : nsISupports
{
  /**
   * display
   *   UI shown when a user is asked to do SSL client auth.
   */
  void ChooseCertificate(in nsIInterfaceRequestor ctx, 
                        in wstring cn,
                        in wstring organization,
                        in wstring issuer,
                        [array, size_is(count)] in wstring certNickList,
                        [array, size_is(count)] in wstring certDetailsList,
                        in unsigned long count,
                        out long selectedIndex,
                        out boolean canceled);
};

%{C++
#define NS_CLIENTAUTHDIALOGS_CONTRACTID "@mozilla.org/nsClientAuthDialogs;1"
%}
