/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM inIScreenCapturer.idl
 */

#ifndef __gen_inIScreenCapturer_h__
#define __gen_inIScreenCapturer_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIDOMWindowInternal; /* forward declaration */

class nsIDOMElement; /* forward declaration */

class inIBitmap; /* forward declaration */


/* starting interface:    inIScreenCapturer */
#define INISCREENCAPTURER_IID_STR "18ee2f91-99f8-49c6-8638-6474d1e64f2d"

#define INISCREENCAPTURER_IID \
  {0x18ee2f91, 0x99f8, 0x49c6, \
    { 0x86, 0x38, 0x64, 0x74, 0xd1, 0xe6, 0x4f, 0x2d }}

class NS_NO_VTABLE inIScreenCapturer : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(INISCREENCAPTURER_IID)

  /* inIBitmap captureElement (in nsIDOMElement aElement); */
  NS_IMETHOD CaptureElement(nsIDOMElement *aElement, inIBitmap **_retval) = 0;

  /* inIBitmap captureRegion (in nsIDOMWindowInternal aWindow, in long aX, in long aY, in long aWidth, in long aHeight); */
  NS_IMETHOD CaptureRegion(nsIDOMWindowInternal *aWindow, PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, inIBitmap **_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_INISCREENCAPTURER \
  NS_IMETHOD CaptureElement(nsIDOMElement *aElement, inIBitmap **_retval); \
  NS_IMETHOD CaptureRegion(nsIDOMWindowInternal *aWindow, PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, inIBitmap **_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_INISCREENCAPTURER(_to) \
  NS_IMETHOD CaptureElement(nsIDOMElement *aElement, inIBitmap **_retval) { return _to CaptureElement(aElement, _retval); } \
  NS_IMETHOD CaptureRegion(nsIDOMWindowInternal *aWindow, PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, inIBitmap **_retval) { return _to CaptureRegion(aWindow, aX, aY, aWidth, aHeight, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_INISCREENCAPTURER(_to) \
  NS_IMETHOD CaptureElement(nsIDOMElement *aElement, inIBitmap **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->CaptureElement(aElement, _retval); } \
  NS_IMETHOD CaptureRegion(nsIDOMWindowInternal *aWindow, PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, inIBitmap **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->CaptureRegion(aWindow, aX, aY, aWidth, aHeight, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class inScreenCapturer : public inIScreenCapturer
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_INISCREENCAPTURER

  inScreenCapturer();
  virtual ~inScreenCapturer();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(inScreenCapturer, inIScreenCapturer)

inScreenCapturer::inScreenCapturer()
{
  /* member initializers and constructor code */
}

inScreenCapturer::~inScreenCapturer()
{
  /* destructor code */
}

/* inIBitmap captureElement (in nsIDOMElement aElement); */
NS_IMETHODIMP inScreenCapturer::CaptureElement(nsIDOMElement *aElement, inIBitmap **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* inIBitmap captureRegion (in nsIDOMWindowInternal aWindow, in long aX, in long aY, in long aWidth, in long aHeight); */
NS_IMETHODIMP inScreenCapturer::CaptureRegion(nsIDOMWindowInternal *aWindow, PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, inIBitmap **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_inIScreenCapturer_h__ */
