/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIURILoader.idl
 */

#ifndef __gen_nsIURILoader_h__
#define __gen_nsIURILoader_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIURIContentListener; /* forward declaration */

class nsIURI; /* forward declaration */

class nsILoadGroup; /* forward declaration */

class nsIDocumentLoader; /* forward declaration */

class nsIProgressEventSink; /* forward declaration */

class nsIChannel; /* forward declaration */

class nsIRequest; /* forward declaration */

class nsIStreamListener; /* forward declaration */

class nsIInputStream; /* forward declaration */


/* starting interface:    nsIURILoader */
#define NS_IURILOADER_IID_STR "40aecb53-8b65-11d3-989d-001083010e9b"

#define NS_IURILOADER_IID \
  {0x40aecb53, 0x8b65, 0x11d3, \
    { 0x98, 0x9d, 0x00, 0x10, 0x83, 0x01, 0x0e, 0x9b }}

class NS_NO_VTABLE nsIURILoader : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IURILOADER_IID)

  /* void registerContentListener (in nsIURIContentListener aContentListener); */
  NS_IMETHOD RegisterContentListener(nsIURIContentListener *aContentListener) = 0;

  /* void unRegisterContentListener (in nsIURIContentListener aContentListener); */
  NS_IMETHOD UnRegisterContentListener(nsIURIContentListener *aContentListener) = 0;

  /* void openURI (in nsIChannel aChannel, in boolean aIsContentPreferred, in nsISupports aWindowContext); */
  NS_IMETHOD OpenURI(nsIChannel *aChannel, PRBool aIsContentPreferred, nsISupports *aWindowContext) = 0;

  /* void stop (in nsISupports aLoadCookie); */
  NS_IMETHOD Stop(nsISupports *aLoadCookie) = 0;

  /* nsILoadGroup getLoadGroupForContext (in nsISupports aWindowContext); */
  NS_IMETHOD GetLoadGroupForContext(nsISupports *aWindowContext, nsILoadGroup **_retval) = 0;

  /* nsIDocumentLoader getDocumentLoaderForContext (in nsISupports aWindowContext); */
  NS_IMETHOD GetDocumentLoaderForContext(nsISupports *aWindowContext, nsIDocumentLoader **_retval) = 0;

  /* void dispatchContent (in string aContentType, in boolean aIsContentPreferred, in nsIRequest aRequest, in nsISupports aCtxt, in nsIURIContentListener aContentListener, in nsISupports aSrcWindowContext, out string aDesiredContentType, out nsIURIContentListener aTargetListener, out boolean abortDispatch); */
  NS_IMETHOD DispatchContent(const char *aContentType, PRBool aIsContentPreferred, nsIRequest *aRequest, nsISupports *aCtxt, nsIURIContentListener *aContentListener, nsISupports *aSrcWindowContext, char **aDesiredContentType, nsIURIContentListener **aTargetListener, PRBool *abortDispatch) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIURILOADER \
  NS_IMETHOD RegisterContentListener(nsIURIContentListener *aContentListener); \
  NS_IMETHOD UnRegisterContentListener(nsIURIContentListener *aContentListener); \
  NS_IMETHOD OpenURI(nsIChannel *aChannel, PRBool aIsContentPreferred, nsISupports *aWindowContext); \
  NS_IMETHOD Stop(nsISupports *aLoadCookie); \
  NS_IMETHOD GetLoadGroupForContext(nsISupports *aWindowContext, nsILoadGroup **_retval); \
  NS_IMETHOD GetDocumentLoaderForContext(nsISupports *aWindowContext, nsIDocumentLoader **_retval); \
  NS_IMETHOD DispatchContent(const char *aContentType, PRBool aIsContentPreferred, nsIRequest *aRequest, nsISupports *aCtxt, nsIURIContentListener *aContentListener, nsISupports *aSrcWindowContext, char **aDesiredContentType, nsIURIContentListener **aTargetListener, PRBool *abortDispatch); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIURILOADER(_to) \
  NS_IMETHOD RegisterContentListener(nsIURIContentListener *aContentListener) { return _to RegisterContentListener(aContentListener); } \
  NS_IMETHOD UnRegisterContentListener(nsIURIContentListener *aContentListener) { return _to UnRegisterContentListener(aContentListener); } \
  NS_IMETHOD OpenURI(nsIChannel *aChannel, PRBool aIsContentPreferred, nsISupports *aWindowContext) { return _to OpenURI(aChannel, aIsContentPreferred, aWindowContext); } \
  NS_IMETHOD Stop(nsISupports *aLoadCookie) { return _to Stop(aLoadCookie); } \
  NS_IMETHOD GetLoadGroupForContext(nsISupports *aWindowContext, nsILoadGroup **_retval) { return _to GetLoadGroupForContext(aWindowContext, _retval); } \
  NS_IMETHOD GetDocumentLoaderForContext(nsISupports *aWindowContext, nsIDocumentLoader **_retval) { return _to GetDocumentLoaderForContext(aWindowContext, _retval); } \
  NS_IMETHOD DispatchContent(const char *aContentType, PRBool aIsContentPreferred, nsIRequest *aRequest, nsISupports *aCtxt, nsIURIContentListener *aContentListener, nsISupports *aSrcWindowContext, char **aDesiredContentType, nsIURIContentListener **aTargetListener, PRBool *abortDispatch) { return _to DispatchContent(aContentType, aIsContentPreferred, aRequest, aCtxt, aContentListener, aSrcWindowContext, aDesiredContentType, aTargetListener, abortDispatch); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIURILOADER(_to) \
  NS_IMETHOD RegisterContentListener(nsIURIContentListener *aContentListener) { return !_to ? NS_ERROR_NULL_POINTER : _to->RegisterContentListener(aContentListener); } \
  NS_IMETHOD UnRegisterContentListener(nsIURIContentListener *aContentListener) { return !_to ? NS_ERROR_NULL_POINTER : _to->UnRegisterContentListener(aContentListener); } \
  NS_IMETHOD OpenURI(nsIChannel *aChannel, PRBool aIsContentPreferred, nsISupports *aWindowContext) { return !_to ? NS_ERROR_NULL_POINTER : _to->OpenURI(aChannel, aIsContentPreferred, aWindowContext); } \
  NS_IMETHOD Stop(nsISupports *aLoadCookie) { return !_to ? NS_ERROR_NULL_POINTER : _to->Stop(aLoadCookie); } \
  NS_IMETHOD GetLoadGroupForContext(nsISupports *aWindowContext, nsILoadGroup **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLoadGroupForContext(aWindowContext, _retval); } \
  NS_IMETHOD GetDocumentLoaderForContext(nsISupports *aWindowContext, nsIDocumentLoader **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetDocumentLoaderForContext(aWindowContext, _retval); } \
  NS_IMETHOD DispatchContent(const char *aContentType, PRBool aIsContentPreferred, nsIRequest *aRequest, nsISupports *aCtxt, nsIURIContentListener *aContentListener, nsISupports *aSrcWindowContext, char **aDesiredContentType, nsIURIContentListener **aTargetListener, PRBool *abortDispatch) { return !_to ? NS_ERROR_NULL_POINTER : _to->DispatchContent(aContentType, aIsContentPreferred, aRequest, aCtxt, aContentListener, aSrcWindowContext, aDesiredContentType, aTargetListener, abortDispatch); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsURILoader : public nsIURILoader
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIURILOADER

  nsURILoader();
  virtual ~nsURILoader();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsURILoader, nsIURILoader)

nsURILoader::nsURILoader()
{
  /* member initializers and constructor code */
}

nsURILoader::~nsURILoader()
{
  /* destructor code */
}

/* void registerContentListener (in nsIURIContentListener aContentListener); */
NS_IMETHODIMP nsURILoader::RegisterContentListener(nsIURIContentListener *aContentListener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void unRegisterContentListener (in nsIURIContentListener aContentListener); */
NS_IMETHODIMP nsURILoader::UnRegisterContentListener(nsIURIContentListener *aContentListener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void openURI (in nsIChannel aChannel, in boolean aIsContentPreferred, in nsISupports aWindowContext); */
NS_IMETHODIMP nsURILoader::OpenURI(nsIChannel *aChannel, PRBool aIsContentPreferred, nsISupports *aWindowContext)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void stop (in nsISupports aLoadCookie); */
NS_IMETHODIMP nsURILoader::Stop(nsISupports *aLoadCookie)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsILoadGroup getLoadGroupForContext (in nsISupports aWindowContext); */
NS_IMETHODIMP nsURILoader::GetLoadGroupForContext(nsISupports *aWindowContext, nsILoadGroup **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIDocumentLoader getDocumentLoaderForContext (in nsISupports aWindowContext); */
NS_IMETHODIMP nsURILoader::GetDocumentLoaderForContext(nsISupports *aWindowContext, nsIDocumentLoader **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void dispatchContent (in string aContentType, in boolean aIsContentPreferred, in nsIRequest aRequest, in nsISupports aCtxt, in nsIURIContentListener aContentListener, in nsISupports aSrcWindowContext, out string aDesiredContentType, out nsIURIContentListener aTargetListener, out boolean abortDispatch); */
NS_IMETHODIMP nsURILoader::DispatchContent(const char *aContentType, PRBool aIsContentPreferred, nsIRequest *aRequest, nsISupports *aCtxt, nsIURIContentListener *aContentListener, nsISupports *aSrcWindowContext, char **aDesiredContentType, nsIURIContentListener **aTargetListener, PRBool *abortDispatch)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIURILoader_h__ */
