/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIKeyedStreamGenerator.idl
 */

#ifndef __gen_nsIKeyedStreamGenerator_h__
#define __gen_nsIKeyedStreamGenerator_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIKeyedStreamGenerator */
#define NS_IKEYEDSTREAMGENERATOR_IID_STR "746bdcde-1dd2-11b2-b114-bf359be651fd"

#define NS_IKEYEDSTREAMGENERATOR_IID \
  {0x746bdcde, 0x1dd2, 0x11b2, \
    { 0xb1, 0x14, 0xbf, 0x35, 0x9b, 0xe6, 0x51, 0xfd }}

class NS_NO_VTABLE nsIKeyedStreamGenerator : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IKEYEDSTREAMGENERATOR_IID)

  /* readonly attribute string signature; */
  NS_IMETHOD GetSignature(char * *aSignature) = 0;

  /* void setup (in unsigned long salt, in nsISupports consumer); */
  NS_IMETHOD Setup(PRUint32 salt, nsISupports *consumer) = 0;

  /* readonly attribute float level; */
  NS_IMETHOD GetLevel(float *aLevel) = 0;

  /* octet getByte (in unsigned long offset); */
  NS_IMETHOD GetByte(PRUint32 offset, PRUint8 *_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIKEYEDSTREAMGENERATOR \
  NS_IMETHOD GetSignature(char * *aSignature); \
  NS_IMETHOD Setup(PRUint32 salt, nsISupports *consumer); \
  NS_IMETHOD GetLevel(float *aLevel); \
  NS_IMETHOD GetByte(PRUint32 offset, PRUint8 *_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIKEYEDSTREAMGENERATOR(_to) \
  NS_IMETHOD GetSignature(char * *aSignature) { return _to GetSignature(aSignature); } \
  NS_IMETHOD Setup(PRUint32 salt, nsISupports *consumer) { return _to Setup(salt, consumer); } \
  NS_IMETHOD GetLevel(float *aLevel) { return _to GetLevel(aLevel); } \
  NS_IMETHOD GetByte(PRUint32 offset, PRUint8 *_retval) { return _to GetByte(offset, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIKEYEDSTREAMGENERATOR(_to) \
  NS_IMETHOD GetSignature(char * *aSignature) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSignature(aSignature); } \
  NS_IMETHOD Setup(PRUint32 salt, nsISupports *consumer) { return !_to ? NS_ERROR_NULL_POINTER : _to->Setup(salt, consumer); } \
  NS_IMETHOD GetLevel(float *aLevel) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLevel(aLevel); } \
  NS_IMETHOD GetByte(PRUint32 offset, PRUint8 *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetByte(offset, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsKeyedStreamGenerator : public nsIKeyedStreamGenerator
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIKEYEDSTREAMGENERATOR

  nsKeyedStreamGenerator();
  virtual ~nsKeyedStreamGenerator();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsKeyedStreamGenerator, nsIKeyedStreamGenerator)

nsKeyedStreamGenerator::nsKeyedStreamGenerator()
{
  /* member initializers and constructor code */
}

nsKeyedStreamGenerator::~nsKeyedStreamGenerator()
{
  /* destructor code */
}

/* readonly attribute string signature; */
NS_IMETHODIMP nsKeyedStreamGenerator::GetSignature(char * *aSignature)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void setup (in unsigned long salt, in nsISupports consumer); */
NS_IMETHODIMP nsKeyedStreamGenerator::Setup(PRUint32 salt, nsISupports *consumer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute float level; */
NS_IMETHODIMP nsKeyedStreamGenerator::GetLevel(float *aLevel)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* octet getByte (in unsigned long offset); */
NS_IMETHODIMP nsKeyedStreamGenerator::GetByte(PRUint32 offset, PRUint8 *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIKeyedStreamGenerator_h__ */
