/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIImportMail.idl
 */

#ifndef __gen_nsIImportMail_h__
#define __gen_nsIImportMail_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIFileSpec; /* forward declaration */

class nsISupportsArray; /* forward declaration */

class nsIImportMailboxDescriptor; /* forward declaration */

class nsIOutputStream; /* forward declaration */


/* starting interface:    nsIImportMail */
#define NS_IIMPORTMAIL_IID_STR "c9e314c0-234e-11d3-a206-00a0cc26da63"

#define NS_IIMPORTMAIL_IID \
  {0xc9e314c0, 0x234e, 0x11d3, \
    { 0xa2, 0x06, 0x00, 0xa0, 0xcc, 0x26, 0xda, 0x63 }}

class NS_NO_VTABLE nsIImportMail : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IIMPORTMAIL_IID)

  /* void GetDefaultLocation (out nsIFileSpec location, out boolean found, out boolean userVerify); */
  NS_IMETHOD GetDefaultLocation(nsIFileSpec **location, PRBool *found, PRBool *userVerify) = 0;

  /* nsISupportsArray FindMailboxes (in nsIFileSpec location); */
  NS_IMETHOD FindMailboxes(nsIFileSpec *location, nsISupportsArray **_retval) = 0;

  /* void ImportMailbox (in nsIImportMailboxDescriptor source, in nsIFileSpec destination, out wstring errorLog, out wstring successLog, out boolean fatalError); */
  NS_IMETHOD ImportMailbox(nsIImportMailboxDescriptor *source, nsIFileSpec *destination, PRUnichar **errorLog, PRUnichar **successLog, PRBool *fatalError) = 0;

  /* unsigned long GetImportProgress (); */
  NS_IMETHOD GetImportProgress(PRUint32 *_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIIMPORTMAIL \
  NS_IMETHOD GetDefaultLocation(nsIFileSpec **location, PRBool *found, PRBool *userVerify); \
  NS_IMETHOD FindMailboxes(nsIFileSpec *location, nsISupportsArray **_retval); \
  NS_IMETHOD ImportMailbox(nsIImportMailboxDescriptor *source, nsIFileSpec *destination, PRUnichar **errorLog, PRUnichar **successLog, PRBool *fatalError); \
  NS_IMETHOD GetImportProgress(PRUint32 *_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIIMPORTMAIL(_to) \
  NS_IMETHOD GetDefaultLocation(nsIFileSpec **location, PRBool *found, PRBool *userVerify) { return _to GetDefaultLocation(location, found, userVerify); } \
  NS_IMETHOD FindMailboxes(nsIFileSpec *location, nsISupportsArray **_retval) { return _to FindMailboxes(location, _retval); } \
  NS_IMETHOD ImportMailbox(nsIImportMailboxDescriptor *source, nsIFileSpec *destination, PRUnichar **errorLog, PRUnichar **successLog, PRBool *fatalError) { return _to ImportMailbox(source, destination, errorLog, successLog, fatalError); } \
  NS_IMETHOD GetImportProgress(PRUint32 *_retval) { return _to GetImportProgress(_retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIIMPORTMAIL(_to) \
  NS_IMETHOD GetDefaultLocation(nsIFileSpec **location, PRBool *found, PRBool *userVerify) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetDefaultLocation(location, found, userVerify); } \
  NS_IMETHOD FindMailboxes(nsIFileSpec *location, nsISupportsArray **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->FindMailboxes(location, _retval); } \
  NS_IMETHOD ImportMailbox(nsIImportMailboxDescriptor *source, nsIFileSpec *destination, PRUnichar **errorLog, PRUnichar **successLog, PRBool *fatalError) { return !_to ? NS_ERROR_NULL_POINTER : _to->ImportMailbox(source, destination, errorLog, successLog, fatalError); } \
  NS_IMETHOD GetImportProgress(PRUint32 *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetImportProgress(_retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsImportMail : public nsIImportMail
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIIMPORTMAIL

  nsImportMail();
  virtual ~nsImportMail();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsImportMail, nsIImportMail)

nsImportMail::nsImportMail()
{
  /* member initializers and constructor code */
}

nsImportMail::~nsImportMail()
{
  /* destructor code */
}

/* void GetDefaultLocation (out nsIFileSpec location, out boolean found, out boolean userVerify); */
NS_IMETHODIMP nsImportMail::GetDefaultLocation(nsIFileSpec **location, PRBool *found, PRBool *userVerify)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsISupportsArray FindMailboxes (in nsIFileSpec location); */
NS_IMETHODIMP nsImportMail::FindMailboxes(nsIFileSpec *location, nsISupportsArray **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void ImportMailbox (in nsIImportMailboxDescriptor source, in nsIFileSpec destination, out wstring errorLog, out wstring successLog, out boolean fatalError); */
NS_IMETHODIMP nsImportMail::ImportMailbox(nsIImportMailboxDescriptor *source, nsIFileSpec *destination, PRUnichar **errorLog, PRUnichar **successLog, PRBool *fatalError)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* unsigned long GetImportProgress (); */
NS_IMETHODIMP nsImportMail::GetImportProgress(PRUint32 *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIImportMail_h__ */
