/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIExternalHelperAppService.idl
 */

#ifndef __gen_nsIExternalHelperAppService_h__
#define __gen_nsIExternalHelperAppService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIURI; /* forward declaration */

class nsIRequest; /* forward declaration */

class nsIStreamListener; /* forward declaration */

class nsIFile; /* forward declaration */

class nsIMIMEInfo; /* forward declaration */

class nsIWebProgressListener; /* forward declaration */


/* starting interface:    nsIExternalHelperAppService */
#define NS_IEXTERNALHELPERAPPSERVICE_IID_STR "663cc0aa-42ea-11d4-98d0-001083010e9b"

#define NS_IEXTERNALHELPERAPPSERVICE_IID \
  {0x663cc0aa, 0x42ea, 0x11d4, \
    { 0x98, 0xd0, 0x00, 0x10, 0x83, 0x01, 0x0e, 0x9b }}

class NS_NO_VTABLE nsIExternalHelperAppService : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IEXTERNALHELPERAPPSERVICE_IID)

  /* boolean canHandleContent (in string aMimeContentType, in nsIURI aUri); */
  NS_IMETHOD CanHandleContent(const char *aMimeContentType, nsIURI *aUri, PRBool *_retval) = 0;

  /* nsIStreamListener doContent (in string aMimeContentType, in nsIRequest aRequest, in nsISupports aWindowContext); */
  NS_IMETHOD DoContent(const char *aMimeContentType, nsIRequest *aRequest, nsISupports *aWindowContext, nsIStreamListener **_retval) = 0;

  /* boolean applyDecodingForType (in string aMimeContentType); */
  NS_IMETHOD ApplyDecodingForType(const char *aMimeContentType, PRBool *_retval) = 0;

  /* boolean applyDecodingForExtension (in string aExtension); */
  NS_IMETHOD ApplyDecodingForExtension(const char *aExtension, PRBool *_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIEXTERNALHELPERAPPSERVICE \
  NS_IMETHOD CanHandleContent(const char *aMimeContentType, nsIURI *aUri, PRBool *_retval); \
  NS_IMETHOD DoContent(const char *aMimeContentType, nsIRequest *aRequest, nsISupports *aWindowContext, nsIStreamListener **_retval); \
  NS_IMETHOD ApplyDecodingForType(const char *aMimeContentType, PRBool *_retval); \
  NS_IMETHOD ApplyDecodingForExtension(const char *aExtension, PRBool *_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIEXTERNALHELPERAPPSERVICE(_to) \
  NS_IMETHOD CanHandleContent(const char *aMimeContentType, nsIURI *aUri, PRBool *_retval) { return _to CanHandleContent(aMimeContentType, aUri, _retval); } \
  NS_IMETHOD DoContent(const char *aMimeContentType, nsIRequest *aRequest, nsISupports *aWindowContext, nsIStreamListener **_retval) { return _to DoContent(aMimeContentType, aRequest, aWindowContext, _retval); } \
  NS_IMETHOD ApplyDecodingForType(const char *aMimeContentType, PRBool *_retval) { return _to ApplyDecodingForType(aMimeContentType, _retval); } \
  NS_IMETHOD ApplyDecodingForExtension(const char *aExtension, PRBool *_retval) { return _to ApplyDecodingForExtension(aExtension, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIEXTERNALHELPERAPPSERVICE(_to) \
  NS_IMETHOD CanHandleContent(const char *aMimeContentType, nsIURI *aUri, PRBool *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->CanHandleContent(aMimeContentType, aUri, _retval); } \
  NS_IMETHOD DoContent(const char *aMimeContentType, nsIRequest *aRequest, nsISupports *aWindowContext, nsIStreamListener **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->DoContent(aMimeContentType, aRequest, aWindowContext, _retval); } \
  NS_IMETHOD ApplyDecodingForType(const char *aMimeContentType, PRBool *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->ApplyDecodingForType(aMimeContentType, _retval); } \
  NS_IMETHOD ApplyDecodingForExtension(const char *aExtension, PRBool *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->ApplyDecodingForExtension(aExtension, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsExternalHelperAppService : public nsIExternalHelperAppService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIEXTERNALHELPERAPPSERVICE

  nsExternalHelperAppService();
  virtual ~nsExternalHelperAppService();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsExternalHelperAppService, nsIExternalHelperAppService)

nsExternalHelperAppService::nsExternalHelperAppService()
{
  /* member initializers and constructor code */
}

nsExternalHelperAppService::~nsExternalHelperAppService()
{
  /* destructor code */
}

/* boolean canHandleContent (in string aMimeContentType, in nsIURI aUri); */
NS_IMETHODIMP nsExternalHelperAppService::CanHandleContent(const char *aMimeContentType, nsIURI *aUri, PRBool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIStreamListener doContent (in string aMimeContentType, in nsIRequest aRequest, in nsISupports aWindowContext); */
NS_IMETHODIMP nsExternalHelperAppService::DoContent(const char *aMimeContentType, nsIRequest *aRequest, nsISupports *aWindowContext, nsIStreamListener **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* boolean applyDecodingForType (in string aMimeContentType); */
NS_IMETHODIMP nsExternalHelperAppService::ApplyDecodingForType(const char *aMimeContentType, PRBool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* boolean applyDecodingForExtension (in string aExtension); */
NS_IMETHODIMP nsExternalHelperAppService::ApplyDecodingForExtension(const char *aExtension, PRBool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsPIExternalAppLauncher */
#define NS_PIEXTERNALAPPLAUNCHER_IID_STR "491d04d5-449c-11d4-98d0-001083010e9b"

#define NS_PIEXTERNALAPPLAUNCHER_IID \
  {0x491d04d5, 0x449c, 0x11d4, \
    { 0x98, 0xd0, 0x00, 0x10, 0x83, 0x01, 0x0e, 0x9b }}

class NS_NO_VTABLE nsPIExternalAppLauncher : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_PIEXTERNALAPPLAUNCHER_IID)

  /* void launchAppWithTempFile (in nsIMIMEInfo aMIMEInfo, in nsIFile aTempFile); */
  NS_IMETHOD LaunchAppWithTempFile(nsIMIMEInfo *aMIMEInfo, nsIFile *aTempFile) = 0;

  /* void deleteTemporaryFileOnExit (in nsIFile aTemporaryFile); */
  NS_IMETHOD DeleteTemporaryFileOnExit(nsIFile *aTemporaryFile) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSPIEXTERNALAPPLAUNCHER \
  NS_IMETHOD LaunchAppWithTempFile(nsIMIMEInfo *aMIMEInfo, nsIFile *aTempFile); \
  NS_IMETHOD DeleteTemporaryFileOnExit(nsIFile *aTemporaryFile); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSPIEXTERNALAPPLAUNCHER(_to) \
  NS_IMETHOD LaunchAppWithTempFile(nsIMIMEInfo *aMIMEInfo, nsIFile *aTempFile) { return _to LaunchAppWithTempFile(aMIMEInfo, aTempFile); } \
  NS_IMETHOD DeleteTemporaryFileOnExit(nsIFile *aTemporaryFile) { return _to DeleteTemporaryFileOnExit(aTemporaryFile); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSPIEXTERNALAPPLAUNCHER(_to) \
  NS_IMETHOD LaunchAppWithTempFile(nsIMIMEInfo *aMIMEInfo, nsIFile *aTempFile) { return !_to ? NS_ERROR_NULL_POINTER : _to->LaunchAppWithTempFile(aMIMEInfo, aTempFile); } \
  NS_IMETHOD DeleteTemporaryFileOnExit(nsIFile *aTemporaryFile) { return !_to ? NS_ERROR_NULL_POINTER : _to->DeleteTemporaryFileOnExit(aTemporaryFile); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class _MYCLASS_ : public nsPIExternalAppLauncher
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSPIEXTERNALAPPLAUNCHER

  _MYCLASS_();
  virtual ~_MYCLASS_();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(_MYCLASS_, nsPIExternalAppLauncher)

_MYCLASS_::_MYCLASS_()
{
  /* member initializers and constructor code */
}

_MYCLASS_::~_MYCLASS_()
{
  /* destructor code */
}

/* void launchAppWithTempFile (in nsIMIMEInfo aMIMEInfo, in nsIFile aTempFile); */
NS_IMETHODIMP _MYCLASS_::LaunchAppWithTempFile(nsIMIMEInfo *aMIMEInfo, nsIFile *aTempFile)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void deleteTemporaryFileOnExit (in nsIFile aTemporaryFile); */
NS_IMETHODIMP _MYCLASS_::DeleteTemporaryFileOnExit(nsIFile *aTemporaryFile)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsIHelperAppLauncher */
#define NS_IHELPERAPPLAUNCHER_IID_STR "9503d0fe-4c9d-11d4-98d0-001083010e9b"

#define NS_IHELPERAPPLAUNCHER_IID \
  {0x9503d0fe, 0x4c9d, 0x11d4, \
    { 0x98, 0xd0, 0x00, 0x10, 0x83, 0x01, 0x0e, 0x9b }}

class NS_NO_VTABLE nsIHelperAppLauncher : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IHELPERAPPLAUNCHER_IID)

  /* readonly attribute nsIMIMEInfo MIMEInfo; */
  NS_IMETHOD GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo) = 0;

  /* readonly attribute nsIURI source; */
  NS_IMETHOD GetSource(nsIURI * *aSource) = 0;

  /* readonly attribute wstring suggestedFileName; */
  NS_IMETHOD GetSuggestedFileName(PRUnichar * *aSuggestedFileName) = 0;

  /* void saveToDisk (in nsIFile aNewFileLocation, in boolean aRememberThisPreference); */
  NS_IMETHOD SaveToDisk(nsIFile *aNewFileLocation, PRBool aRememberThisPreference) = 0;

  /* void launchWithApplication (in nsIFile aApplication, in boolean aRememberThisPreference); */
  NS_IMETHOD LaunchWithApplication(nsIFile *aApplication, PRBool aRememberThisPreference) = 0;

  /* void Cancel (); */
  NS_IMETHOD Cancel(void) = 0;

  /* void setWebProgressListener (in nsIWebProgressListener aWebProgressListener); */
  NS_IMETHOD SetWebProgressListener(nsIWebProgressListener *aWebProgressListener) = 0;

  /* void closeProgressWindow (); */
  NS_IMETHOD CloseProgressWindow(void) = 0;

  /* nsIFile getDownloadInfo (out nsIURI aSourceUrl, out long long aTimeDownloadStarted); */
  NS_IMETHOD GetDownloadInfo(nsIURI **aSourceUrl, PRInt64 *aTimeDownloadStarted, nsIFile **_retval) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIHELPERAPPLAUNCHER \
  NS_IMETHOD GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo); \
  NS_IMETHOD GetSource(nsIURI * *aSource); \
  NS_IMETHOD GetSuggestedFileName(PRUnichar * *aSuggestedFileName); \
  NS_IMETHOD SaveToDisk(nsIFile *aNewFileLocation, PRBool aRememberThisPreference); \
  NS_IMETHOD LaunchWithApplication(nsIFile *aApplication, PRBool aRememberThisPreference); \
  NS_IMETHOD Cancel(void); \
  NS_IMETHOD SetWebProgressListener(nsIWebProgressListener *aWebProgressListener); \
  NS_IMETHOD CloseProgressWindow(void); \
  NS_IMETHOD GetDownloadInfo(nsIURI **aSourceUrl, PRInt64 *aTimeDownloadStarted, nsIFile **_retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIHELPERAPPLAUNCHER(_to) \
  NS_IMETHOD GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo) { return _to GetMIMEInfo(aMIMEInfo); } \
  NS_IMETHOD GetSource(nsIURI * *aSource) { return _to GetSource(aSource); } \
  NS_IMETHOD GetSuggestedFileName(PRUnichar * *aSuggestedFileName) { return _to GetSuggestedFileName(aSuggestedFileName); } \
  NS_IMETHOD SaveToDisk(nsIFile *aNewFileLocation, PRBool aRememberThisPreference) { return _to SaveToDisk(aNewFileLocation, aRememberThisPreference); } \
  NS_IMETHOD LaunchWithApplication(nsIFile *aApplication, PRBool aRememberThisPreference) { return _to LaunchWithApplication(aApplication, aRememberThisPreference); } \
  NS_IMETHOD Cancel(void) { return _to Cancel(); } \
  NS_IMETHOD SetWebProgressListener(nsIWebProgressListener *aWebProgressListener) { return _to SetWebProgressListener(aWebProgressListener); } \
  NS_IMETHOD CloseProgressWindow(void) { return _to CloseProgressWindow(); } \
  NS_IMETHOD GetDownloadInfo(nsIURI **aSourceUrl, PRInt64 *aTimeDownloadStarted, nsIFile **_retval) { return _to GetDownloadInfo(aSourceUrl, aTimeDownloadStarted, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIHELPERAPPLAUNCHER(_to) \
  NS_IMETHOD GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetMIMEInfo(aMIMEInfo); } \
  NS_IMETHOD GetSource(nsIURI * *aSource) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSource(aSource); } \
  NS_IMETHOD GetSuggestedFileName(PRUnichar * *aSuggestedFileName) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSuggestedFileName(aSuggestedFileName); } \
  NS_IMETHOD SaveToDisk(nsIFile *aNewFileLocation, PRBool aRememberThisPreference) { return !_to ? NS_ERROR_NULL_POINTER : _to->SaveToDisk(aNewFileLocation, aRememberThisPreference); } \
  NS_IMETHOD LaunchWithApplication(nsIFile *aApplication, PRBool aRememberThisPreference) { return !_to ? NS_ERROR_NULL_POINTER : _to->LaunchWithApplication(aApplication, aRememberThisPreference); } \
  NS_IMETHOD Cancel(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->Cancel(); } \
  NS_IMETHOD SetWebProgressListener(nsIWebProgressListener *aWebProgressListener) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetWebProgressListener(aWebProgressListener); } \
  NS_IMETHOD CloseProgressWindow(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->CloseProgressWindow(); } \
  NS_IMETHOD GetDownloadInfo(nsIURI **aSourceUrl, PRInt64 *aTimeDownloadStarted, nsIFile **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetDownloadInfo(aSourceUrl, aTimeDownloadStarted, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsHelperAppLauncher : public nsIHelperAppLauncher
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIHELPERAPPLAUNCHER

  nsHelperAppLauncher();
  virtual ~nsHelperAppLauncher();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsHelperAppLauncher, nsIHelperAppLauncher)

nsHelperAppLauncher::nsHelperAppLauncher()
{
  /* member initializers and constructor code */
}

nsHelperAppLauncher::~nsHelperAppLauncher()
{
  /* destructor code */
}

/* readonly attribute nsIMIMEInfo MIMEInfo; */
NS_IMETHODIMP nsHelperAppLauncher::GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIURI source; */
NS_IMETHODIMP nsHelperAppLauncher::GetSource(nsIURI * *aSource)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute wstring suggestedFileName; */
NS_IMETHODIMP nsHelperAppLauncher::GetSuggestedFileName(PRUnichar * *aSuggestedFileName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void saveToDisk (in nsIFile aNewFileLocation, in boolean aRememberThisPreference); */
NS_IMETHODIMP nsHelperAppLauncher::SaveToDisk(nsIFile *aNewFileLocation, PRBool aRememberThisPreference)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void launchWithApplication (in nsIFile aApplication, in boolean aRememberThisPreference); */
NS_IMETHODIMP nsHelperAppLauncher::LaunchWithApplication(nsIFile *aApplication, PRBool aRememberThisPreference)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void Cancel (); */
NS_IMETHODIMP nsHelperAppLauncher::Cancel()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void setWebProgressListener (in nsIWebProgressListener aWebProgressListener); */
NS_IMETHODIMP nsHelperAppLauncher::SetWebProgressListener(nsIWebProgressListener *aWebProgressListener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void closeProgressWindow (); */
NS_IMETHODIMP nsHelperAppLauncher::CloseProgressWindow()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIFile getDownloadInfo (out nsIURI aSourceUrl, out long long aTimeDownloadStarted); */
NS_IMETHODIMP nsHelperAppLauncher::GetDownloadInfo(nsIURI **aSourceUrl, PRInt64 *aTimeDownloadStarted, nsIFile **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIExternalHelperAppService_h__ */
